#!/usr/bin/env python

import time, sys

import dnet
sys.path.insert(0, '.')
import dpkt
from impacket import ImpactDecoder, ImpactPacket
from openbsd import packet
import scapy
import xstruct

xip = xstruct.structdef('>', [
    ('v_hl', ('B', 1), (4 << 4) | (dnet.IP_HDR_LEN >> 2)),
    ('tos', ('B', 1), dnet.IP_TOS_DEFAULT),
    ('len', ('H', 1), dnet.IP_HDR_LEN),
    ('id', ('H', 1), 0),
    ('off', ('H', 1), 0),
    ('ttl', ('B', 1), dnet.IP_TTL_DEFAULT),
    ('p', ('B', 1), 0),
    ('sum', ('H', 1), 0),
    ('src', ('s', dnet.IP_ADDR_LEN), dnet.IP_ADDR_ANY),
    ('dst', ('s', dnet.IP_ADDR_LEN), dnet.IP_ADDR_ANY)
    ])

xudp = xstruct.structdef('>', [
    ('sport', ('B', 1), 0),
    ('dport', ('B', 1), 0),
    ('ulen', ('H', 1), dnet.UDP_HDR_LEN),
    ('sum', ('H', 1), 0)
    ])

def compare_create(cnt):
    """
dpkt: 14915.2445937 pps
dpkt (manual): 15494.3632903 pps
impacket: 3929.30572776 pps
openbsd.packet: 1503.7928579 pps
scapy: 348.449269721 pps
xstruct: 88314.8953732 pps
"""
    src = dnet.addr('1.2.3.4').ip
    dst = dnet.addr('5.6.7.8').ip
    data = 'hello world'

    start = time.time()
    for i in xrange(cnt):
        dnet.ip_checksum(
            str(dpkt.ip.IP(src=src, dst=dst, p=dnet.IP_PROTO_UDP,
                         len = dnet.IP_HDR_LEN + dnet.UDP_HDR_LEN + len(data),
                         data=dpkt.udp.UDP(sport=111, dport=222,
                                       ulen=dnet.UDP_HDR_LEN + len(data),
                                       data=data))))
    print 'dpkt:', cnt / (time.time() - start), 'pps'
    
    start = time.time()
    for i in xrange(cnt):
        dnet.ip_checksum(str(dpkt.ip.IP(src=src, dst=dst, p=dnet.IP_PROTO_UDP,
                                     len=dnet.IP_HDR_LEN + dnet.UDP_HDR_LEN +
                                     len(data))) +
                         str(dpkt.udp.UDP(sport=111, dport=222,
                                      ulen=dnet.UDP_HDR_LEN + len(data))) +
                         data)
    print 'dpkt (manual):', cnt / (time.time() - start), 'pps'
    
    start = time.time()
    for i in xrange(cnt):
        ip = ImpactPacket.IP()
        ip.set_ip_src('1.2.3.4')
        ip.set_ip_dst('5.6.7.8')
        udp = ImpactPacket.UDP()
        udp.set_uh_sport(111)
        udp.set_uh_dport(222)
        udp.contains(ImpactPacket.Data(data))
        ip.contains(udp)
        ip.get_packet()
    print 'impacket:', cnt / (time.time() - start), 'pps'

    start = time.time()
    for i in xrange(cnt):
        p = packet.createPacket(packet.IP, packet.UDP)
        p['ip'].src = '1.2.3.4'
        p['ip'].dst = '5.6.7.8'
        p['udp'].sport = 111
        p['udp'].dport = 22
        p['udp'].payload = data
        p.finalise()
        p.getRaw()
    print 'openbsd.packet:', cnt / (time.time() - start), 'pps'
    
    start = time.time()
    for i in xrange(cnt):
        ip = scapy.IP(src='1.2.3.4', dst='5.6.7.8') / \
             scapy.UDP(sport=111, dport=222) / data
        ip.build()
    print 'scapy:', cnt / (time.time() - start), 'pps'
    
    start = time.time()
    for i in xrange(cnt):
        udp = xudp()
        udp.sport = 111
        udp.dport = 222
        udp.ulen = dnet.UDP_HDR_LEN + len(data)
        ip = xip()
        ip.src = src
        ip.dst = dst
        ip.p = dnet.IP_PROTO_UDP
        ip.len = dnet.IP_HDR_LEN + udp.ulen
        dnet.ip_checksum(str(ip) + str(udp) + data)
    print 'xstruct:', cnt / (time.time() - start), 'pps'
    
def compare_parse(cnt):
    """
dpkt: 23347.462887 pps
impacket: 9937.75963595 pps
openbsd.packet: 6826.5955563 pps
scapy: 1461.74727127 pps
xstruct: 206100.202449 pps
"""
    s = 'E\x00\x00T\xc2\xf3\x00\x00\xff\x01\xe2\x18\n\x00\x01\x92\n\x00\x01\x0b\x08\x00\xfc\x11:g\x00\x00A,\xc66\x00\x0e\xcf\x12\x08\t\n\x0b\x0c\r\x0e\x0f\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f!"#$%&\'()*+,-./01234567'

    start = time.time()
    for i in xrange(cnt):
        dpkt.ip.IP(s)
    print 'dpkt:', cnt / (time.time() - start), 'pps'
    
    decoder = ImpactDecoder.IPDecoder()
    start = time.time()
    for i in xrange(cnt):
        decoder.decode(s)
    print 'impacket:', cnt / (time.time() - start), 'pps'

    start = time.time()
    for i in xrange(cnt):
        packet.Packet(packet.IP, s)
    print 'openbsd.packet:', cnt / (time.time() - start), 'pps'

    start = time.time()
    for i in xrange(cnt):
        scapy.IP(s)
    print 'scapy:', cnt / (time.time() - start), 'pps'

    start = time.time()
    for i in xrange(cnt):
        ip = xip(s[:dnet.IP_HDR_LEN])
        udp = xudp(s[dnet.IP_HDR_LEN:dnet.IP_HDR_LEN + dnet.UDP_HDR_LEN])
        data = s[dnet.IP_HDR_LEN + dnet.UDP_HDR_LEN:]
    print 'xstruct:', cnt / (time.time() - start), 'pps'

def compare_checksum(cnt):
    s = 'A' * 80
    start = time.time()
    for i in range(cnt):
        dpkt.in_cksum(s)
    print 'dpkt.in_cksum:', cnt / (time.time() - start), 'pps'
    
    start = time.time()
    for i in range(cnt):
        dnet.ip_cksum_carry(dnet.ip_cksum_add(s, 0))
    print 'dnet.ip_cksum_add/carry:', cnt / (time.time() - start), 'pps'

def main():
    import psyco
    psyco.full()

    ITER=10000
    
    print 'checksum:'
    compare_checksum(100000)

    print 'create:'
    compare_create(ITER)

    print 'parse:'
    compare_parse(ITER)
    
if __name__ == '__main__':
    main()
    """
    import hotshot, hotshot.stats
    prof = hotshot.Profile('/var/tmp/dpkt.prof')
    prof.runcall(main)
    prof.close()
    stats = hotshot.stats.load('/var/tmp/dpkt.prof')
    stats.strip_dirs()
    stats.sort_stats('time', 'calls')
    stats.print_stats(20)
    """



# $Id: loopback.py 38 2007-03-17 03:33:16Z dugsong $

"""Platform-dependent loopback header."""

import dpkt, ethernet, ip, ip6

class Loopback(dpkt.Packet):
    __hdr__ = (('family', 'I', 0), )
    __byte_order__ = '@'
    def unpack(self, buf):
        dpkt.Packet.unpack(self, buf)
        if self.family == 2:
            self.data = ip.IP(self.data)
        elif self.family == 0x02000000:
            self.family = 2
            self.data = ip.IP(self.data)
        elif self.family in (24, 28, 30):
            self.data = ip6.IP6(self.data)
        elif self.family > 1500:
            self.data = ethernet.Ethernet(self.data)



#!/usr/bin/env python

import dpkt
import time, unittest

class TestPerf(unittest.TestCase):
    rounds = 10000

    def setUp(self):
        self.start = time.time()
    def tearDown(self):
        print self.rounds / (time.time() - self.start), 'rounds/s'

    def test_pack(self):
        for i in xrange(self.rounds):
            str(dpkt.ip.IP())
        print 'pack:',

    def test_unpack(self):
        buf = str(dpkt.ip.IP())
        for i in xrange(self.rounds):
            dpkt.ip.IP(buf)
        print 'unpack:',
        
if __name__ == '__main__':
    unittest.main()



# $Id: pim.py 23 2006-11-08 15:45:33Z dugsong $

"""Protocol Independent Multicast."""

import dpkt

class PIM(dpkt.Packet):
    __hdr__ = (
        ('v_type', 'B', 0x20),
        ('rsvd', 'B', 0),
        ('sum', 'H', 0)
        )
    def _get_v(self): return self.v_type >> 4
    def _set_v(self, v): self.v_type = (v << 4) | (self.v_type & 0xf)
    v = property(_get_v, _set_v)
    
    def _get_type(self): return self.v_type & 0xf
    def _set_type(self, type): self.v_type = (self.v_type & 0xf0) | type
    type = property(_get_type, _set_type)

    def __str__(self):
        if not self.sum:
            self.sum = dpkt.in_cksum(dpkt.Packet.__str__(self))
        return dpkt.Packet.__str__(self)



# $Id: tpkt.py 23 2006-11-08 15:45:33Z dugsong $

"""ISO Transport Service on top of the TCP (TPKT)."""

import dpkt

# TPKT - RFC 1006 Section 6
# http://www.faqs.org/rfcs/rfc1006.html

class TPKT(dpkt.Packet):
    __hdr__ = (
        ('v', 'B', 3),
        ('rsvd', 'B', 0),
        ('len', 'H', 0)
        )



# $Id: rx.py 23 2006-11-08 15:45:33Z jonojono $

"""Rx Protocol."""

import dpkt

# Types
DATA                    = 0x01
ACK                     = 0x02
BUSY                    = 0x03
ABORT                   = 0x04
ACKALL                  = 0x05
CHALLENGE               = 0x06
RESPONSE                = 0x07
DEBUG                   = 0x08

# Flags
CLIENT_INITIATED        = 0x01 
REQUEST_ACK             = 0x02
LAST_PACKET             = 0x04
MORE_PACKETS            = 0x08
SLOW_START_OK           = 0x20
JUMBO_PACKET            = 0x20

# Security
SEC_NONE                = 0x00
SEC_BCRYPT              = 0x01
SEC_RXKAD               = 0x02
SEC_RXKAD_ENC           = 0x03

class Rx(dpkt.Packet):
    __hdr__ = (
        ('epoch', 'I', 0),
        ('cid', 'I', 0),
        ('call', 'I', 1),
        ('seq', 'I', 0),
        ('serial', 'I', 1),
        ('type', 'B', 0),
        ('flags', 'B', CLIENT_INITIATED),
        ('status', 'B', 0),
        ('security', 'B', 0),
        ('sum', 'H', 0),
        ('service', 'H', 0)
        )



# $Id: cdp.py 23 2006-11-08 15:45:33Z dugsong $

"""Cisco Discovery Protocol."""

import struct
import dpkt

CDP_DEVID		= 1	# string
CDP_ADDRESS		= 2
CDP_PORTID		= 3	# string
CDP_CAPABILITIES	= 4	# 32-bit bitmask
CDP_VERSION		= 5	# string
CDP_PLATFORM		= 6	# string
CDP_IPPREFIX		= 7	

CDP_VTP_MGMT_DOMAIN	= 9	# string
CDP_NATIVE_VLAN		= 10	# 16-bit integer
CDP_DUPLEX		= 11	# 8-bit boolean
CDP_TRUST_BITMAP	= 18	# 8-bit bitmask0x13
CDP_UNTRUST_COS		= 19	# 8-bit port
CDP_SYSTEM_NAME		= 20	# string
CDP_SYSTEM_OID		= 21	# 10-byte binary string
CDP_MGMT_ADDRESS	= 22	# 32-bit number of addrs, Addresses
CDP_LOCATION		= 23	# string

class CDP(dpkt.Packet):
    __hdr__ = (
        ('version', 'B', 2),
        ('ttl', 'B', 180),
        ('sum', 'H', 0)
        )
    class Address(dpkt.Packet):
        # XXX - only handle NLPID/IP for now
        __hdr__ = (
            ('ptype', 'B', 1),	# protocol type (NLPID)
            ('plen', 'B', 1),	# protocol length
            ('p', 'B', 0xcc),	# IP
            ('alen', 'H', 4)	# address length
            )
        def unpack(self, buf):
            dpkt.Packet.unpack(self, buf)
            self.data = self.data[:self.alen]
            
    class TLV(dpkt.Packet):
        __hdr__ = (
            ('type', 'H', 0),
            ('len', 'H', 4)
            )
        def unpack(self, buf):
            dpkt.Packet.unpack(self, buf)
            self.data = self.data[:self.len - 4]
            if self.type == CDP_ADDRESS:
                n = struct.unpack('>I', self.data[:4])[0]
                buf = self.data[4:]
                l = []
                for i in range(n):
                    a = CDP.Address(buf)
                    l.append(a)
                    buf = buf[len(a):]
                self.data = l

        def __len__(self):
            if self.type == CDP_ADDRESS:
                n = 4 + sum(map(len, self.data))
            else:
                n = len(self.data)
            return self.__hdr_len__ + n
        
        def __str__(self):
            self.len = len(self)
            if self.type == CDP_ADDRESS:
                s = struct.pack('>I', len(self.data)) + \
                    ''.join(map(str, self.data))
            else:
                s = self.data
            return self.pack_hdr() + s

    def unpack(self, buf):
        dpkt.Packet.unpack(self, buf)
        buf = self.data
        l = []
        while buf:
            tlv = self.TLV(buf)
            l.append(tlv)
            buf = buf[len(tlv):]
        self.data = l

    def __len__(self):
        return self.__hdr_len__ + sum(map(len, self.data))

    def __str__(self):
        data = ''.join(map(str, self.data))
        if not self.sum:
            self.sum = dpkt.in_cksum(self.pack_hdr() + data)
        return self.pack_hdr() + data



#!/usr/bin/env python

import random, socket
import dpkt
import ping

class DNSPing(ping.Ping):
    def __init__(self):
        ping.Ping.__init__(self)
        self.op.add_option('-z', dest='zone', type='string',
                      default=socket.gethostname().split('.', 1)[1],
                      help='Domain to formulate queries in')
        self.op.add_option('-n', dest='hostname', type='string',
                      help='Query only for a given hostname')
        self.op.add_option('-p', dest='port', type='int', default=53,
                      help='Remote DNS server port')
        self.op.add_option('-R', dest='norecurse', action='store_true',
                      help='Disable recursive queries')

    def open_sock(self, opts):
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.connect((opts.ip, opts.port))
        sock.settimeout(opts.wait)
        return sock

    def gen_ping(self, opts):
        for i in xrange(opts.count):
            dns = dpkt.dns.DNS(id=i)
            if opts.norecurse:
                dns.op &= ~dpkt.dns.DNS_RD
            if not opts.hostname:
                name = '%s.%s' % (str(random.random())[-6:], opts.zone)
            else:
                name = opts.hostname
            dns.qd = [ dpkt.dns.DNS.Q(name=name) ]
            yield str(dns)

    def print_header(self, opts):
        print 'DNSPING %s:' % opts.ip,
        if opts.hostname:
            print 'Name: %s' % opts.hostname
        else:
            print 'Name: *.%s' % opts.zone
        
    def print_reply(self, opts, buf, rtt):
        dns = dpkt.dns.DNS(buf)
        print '%d bytes from %s: id=%d time=%.3f ms' % \
              (len(buf), opts.ip, dns.id, rtt * 1000)

if __name__ == '__main__':
    DNSPing().main()



# $Id: dtp.py 23 2006-11-08 15:45:33Z dugsong $

"""Dynamic Trunking Protocol."""

import struct
import dpkt

class DTP(dpkt.Packet):
    __hdr__ = (
        ('v', 'B', 0),
        ) # rest is TLVs
    def unpack(self, buf):
        dpkt.Packet.unpack(self, buf)
        buf = self.data
        tvs = []
        while buf:
            t, l = struct.unpack('>HH', buf[:4])
            v, buf = buf[4:4+l], buf[4+l:]
            tvs.append((t, v))
        self.data = tvs

TRUNK_NAME = 0x01
MAC_ADDR = 0x04



# $Id: pcap.py 77 2011-01-06 15:59:38Z dugsong $

"""Libpcap file format."""

import sys, time
import dpkt

TCPDUMP_MAGIC = 0xa1b2c3d4L
PMUDPCT_MAGIC = 0xd4c3b2a1L

PCAP_VERSION_MAJOR = 2
PCAP_VERSION_MINOR = 4

DLT_NULL               = 0
DLT_EN10MB             = 1
DLT_EN3MB              = 2
DLT_AX25               = 3
DLT_PRONET             = 4
DLT_CHAOS              = 5
DLT_IEEE802            = 6
DLT_ARCNET             = 7
DLT_SLIP               = 8
DLT_PPP                = 9
DLT_FDDI               = 10
DLT_PFSYNC             = 18
DLT_IEEE802_11         = 105
DLT_LINUX_SLL          = 113
DLT_PFLOG              = 117
DLT_IEEE802_11_RADIO   = 127

if sys.platform.find('openbsd') != -1:
    DLT_LOOP           = 12
    DLT_RAW            = 14
else:
    DLT_LOOP           = 108
    DLT_RAW            = 12

dltoff = { DLT_NULL:4, DLT_EN10MB:14, DLT_IEEE802:22, DLT_ARCNET:6,
           DLT_SLIP:16, DLT_PPP:4, DLT_FDDI:21, DLT_PFLOG:48, DLT_PFSYNC:4,
           DLT_LOOP:4, DLT_LINUX_SLL:16 }

class PktHdr(dpkt.Packet):
    """pcap packet header."""
    __hdr__ = (
        ('tv_sec', 'I', 0),
        ('tv_usec', 'I', 0),
        ('caplen', 'I', 0),
        ('len', 'I', 0),
        )

class LEPktHdr(PktHdr):
    __byte_order__ = '<'

class FileHdr(dpkt.Packet):
    """pcap file header."""
    __hdr__ = (
        ('magic', 'I', TCPDUMP_MAGIC),
        ('v_major', 'H', PCAP_VERSION_MAJOR),
        ('v_minor', 'H', PCAP_VERSION_MINOR),
        ('thiszone', 'I', 0),
        ('sigfigs', 'I', 0),
        ('snaplen', 'I', 1500),
        ('linktype', 'I', 1),
        )

class LEFileHdr(FileHdr):
    __byte_order__ = '<'

class Writer(object):
    """Simple pcap dumpfile writer."""
    def __init__(self, fileobj, snaplen=1500, linktype=DLT_EN10MB):
        self.__f = fileobj
        if sys.byteorder == 'little':
            fh = LEFileHdr(snaplen=snaplen, linktype=linktype)
        else:
            fh = FileHdr(snaplen=snaplen, linktype=linktype)
        self.__f.write(str(fh))

    def writepkt(self, pkt, ts=None):
        if ts is None:
            ts = time.time()
        s = str(pkt)
        n = len(s)
        if sys.byteorder == 'little':
            ph = LEPktHdr(tv_sec=int(ts),
                    tv_usec=int((float(ts) - int(ts)) * 1000000.0),
                    caplen=n, len=n)
        else:
            ph = PktHdr(tv_sec=int(ts),
                    tv_usec=int((float(ts) - int(ts)) * 1000000.0),
                    caplen=n, len=n)
        self.__f.write(str(ph))
        self.__f.write(s)

    def close(self):
        self.__f.close()

class Reader(object):
    """Simple pypcap-compatible pcap file reader."""
    
    def __init__(self, fileobj):
        self.name = fileobj.name
        self.fd = fileobj.fileno()
        self.__f = fileobj
        buf = self.__f.read(FileHdr.__hdr_len__)
        self.__fh = FileHdr(buf)
        self.__ph = PktHdr
        if self.__fh.magic == PMUDPCT_MAGIC:
            self.__fh = LEFileHdr(buf)
            self.__ph = LEPktHdr
        elif self.__fh.magic != TCPDUMP_MAGIC:
            raise ValueError, 'invalid tcpdump header'
        if self.__fh.linktype in dltoff:
            self.dloff = dltoff[self.__fh.linktype]
        else:
            self.dloff = 0
        self.snaplen = self.__fh.snaplen
        self.filter = ''

    def fileno(self):
        return self.fd
    
    def datalink(self):
        return self.__fh.linktype
    
    def setfilter(self, value, optimize=1):
        return NotImplementedError

    def readpkts(self):
        return list(self)
    
    def dispatch(self, cnt, callback, *args):
        if cnt > 0:
            for i in range(cnt):
                ts, pkt = self.next()
                callback(ts, pkt, *args)
        else:
            for ts, pkt in self:
                callback(ts, pkt, *args)

    def loop(self, callback, *args):
        self.dispatch(0, callback, *args)
    
    def __iter__(self):
        self.__f.seek(FileHdr.__hdr_len__)
        while 1:
            buf = self.__f.read(PktHdr.__hdr_len__)
            if not buf: break
            hdr = self.__ph(buf)
            buf = self.__f.read(hdr.caplen)
            yield (hdr.tv_sec + (hdr.tv_usec / 1000000.0), buf)

if __name__ == '__main__':
    import unittest

    class PcapTestCase(unittest.TestCase):
        def test_endian(self):
            be = '\xa1\xb2\xc3\xd4\x00\x02\x00\x04\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x60\x00\x00\x00\x01'
            le = '\xd4\xc3\xb2\xa1\x02\x00\x04\x00\x00\x00\x00\x00\x00\x00\x00\x00\x60\x00\x00\x00\x01\x00\x00\x00'
            befh = FileHdr(be)
            lefh = LEFileHdr(le)
            self.failUnless(befh.linktype == lefh.linktype)

    unittest.main()



'''Radiotap'''

import dpkt
import ieee80211
import socket

# Ref: http://www.radiotap.org
# Fields Ref: http://www.radiotap.org/defined-fields/all

# Present flags
_TSFT_MASK            = 0x1000000
_FLAGS_MASK           = 0x2000000
_RATE_MASK            = 0x4000000
_CHANNEL_MASK         = 0x8000000
_FHSS_MASK            = 0x10000000
_ANT_SIG_MASK         = 0x20000000
_ANT_NOISE_MASK       = 0x40000000
_LOCK_QUAL_MASK       = 0x80000000
_TX_ATTN_MASK         = 0x10000
_DB_TX_ATTN_MASK      = 0x20000
_DBM_TX_POWER_MASK    = 0x40000
_ANTENNA_MASK         = 0x80000
_DB_ANT_SIG_MASK      = 0x100000
_DB_ANT_NOISE_MASK    = 0x200000
_RX_FLAGS_MASK        = 0x400000
_CHANNELPLUS_MASK     = 0x200
_EXT_MASK             = 0x1

_TSFT_SHIFT           = 24
_FLAGS_SHIFT          = 25
_RATE_SHIFT           = 26
_CHANNEL_SHIFT        = 27
_FHSS_SHIFT           = 28
_ANT_SIG_SHIFT        = 29
_ANT_NOISE_SHIFT      = 30
_LOCK_QUAL_SHIFT      = 31
_TX_ATTN_SHIFT        = 16
_DB_TX_ATTN_SHIFT     = 17
_DBM_TX_POWER_SHIFT   = 18
_ANTENNA_SHIFT        = 19
_DB_ANT_SIG_SHIFT     = 20
_DB_ANT_NOISE_SHIFT   = 21
_RX_FLAGS_SHIFT       = 22
_CHANNELPLUS_SHIFT    = 10
_EXT_SHIFT            = 0

# Flags elements
_FLAGS_SIZE           = 2
_CFP_FLAG_SHIFT       = 0
_PREAMBLE_SHIFT       = 1
_WEP_SHIFT            = 2
_FRAG_SHIFT           = 3
_FCS_SHIFT            = 4
_DATA_PAD_SHIFT       = 5
_BAD_FCS_SHIFT        = 6
_SHORT_GI_SHIFT       = 7

# Channel type
_CHAN_TYPE_SIZE       = 4
_CHANNEL_TYPE_SHIFT   = 4
_CCK_SHIFT            = 5
_OFDM_SHIFT           = 6
_TWO_GHZ_SHIFT        = 7
_FIVE_GHZ_SHIFT       = 8
_PASSIVE_SHIFT        = 9
_DYN_CCK_OFDM_SHIFT   = 10
_GFSK_SHIFT           = 11
_GSM_SHIFT            = 12
_STATIC_TURBO_SHIFT   = 13
_HALF_RATE_SHIFT      = 14
_QUARTER_RATE_SHIFT   = 15

# Flags offsets and masks
_FCS_SHIFT = 4
_FCS_MASK = 0x10

class Radiotap(dpkt.Packet):
    __hdr__ = (
        ('version', 'B', 0),
        ('pad', 'B', 0),
        ('length', 'H', 0),
        ('present_flags', 'I', 0)
        )

    def _get_tsft_present(self): return (self.present_flags & _TSFT_MASK) >> _TSFT_SHIFT
    def _set_tsft_present(self, val): self.present_flags = self.present_flags | (val << _TSFT_SHIFT)
    def _get_flags_present(self): return (self.present_flags & _FLAGS_MASK) >> _FLAGS_SHIFT
    def _set_flags_present(self, val): self.present_flags = self.present_flags | (val << _FLAGS_SHIFT)
    def _get_rate_present(self): return (self.present_flags & _RATE_MASK) >> _RATE_SHIFT
    def _set_rate_present(self, val): self.present_flags = self.present_flags | (val <<    _RATE_SHIFT)
    def _get_channel_present(self): return (self.present_flags & _CHANNEL_MASK) >> _CHANNEL_SHIFT
    def _set_channel_present(self, val): self.present_flags = self.present_flags | (val << _CHANNEL_SHIFT)
    def _get_fhss_present(self): return (self.present_flags & _FHSS_MASK) >> _FHSS_SHIFT
    def _set_fhss_present(self, val): self.present_flags = self.present_flags | (val << _FHSS_SHIFT)
    def _get_ant_sig_present(self): return (self.present_flags & _ANT_SIG_MASK) >> _ANT_SIG_SHIFT
    def _set_ant_sig_present(self, val): self.present_flags = self.present_flags | (val << _ANT_SIG_SHIFT)
    def _get_ant_noise_present(self): return (self.present_flags & _ANT_NOISE_MASK) >> _ANT_NOISE_SHIFT
    def _set_ant_noise_present(self, val): self.present_flags = self.present_flags | (val << _ANT_NOISE_SHIFT)
    def _get_lock_qual_present(self): return (self.present_flags & _LOCK_QUAL_MASK) >> _LOCK_QUAL_SHIFT
    def _set_lock_qual_present(self, val): self.present_flags = self.present_flags | (val << _LOCK_QUAL_SHIFT)
    def _get_tx_attn_present(self): return (self.present_flags & _TX_ATTN_MASK) >> _TX_ATTN_SHIFT
    def _set_tx_attn_present(self, val): self.present_flags = self.present_flags | (val  << _TX_ATTN_SHIFT)
    def _get_db_tx_attn_present(self): return (self.present_flags & _DB_TX_ATTN_MASK) >> _DB_TX_ATTN_SHIFT
    def _set_db_tx_attn_present(self, val): self.present_flags = self.present_flags | (val << _DB_TX_ATTN_SHIFT)
    def _get_dbm_power_present(self): return (self.present_flags & _DBM_TX_POWER_MASK) >> _DBM_TX_POWER_SHIFT
    def _set_dbm_power_present(self, val): self.present_flags = self.present_flags | (val << _DBM_TX_POWER_SHIFT)
    def _get_ant_present(self): return (self.present_flags & _ANTENNA_MASK) >> _ANTENNA_SHIFT
    def _set_ant_present(self, val): self.present_flags = self.present_flags | (val << _ANTENNA_SHIFT)
    def _get_db_ant_sig_present(self): return (self.present_flags & _DB_ANT_SIG_MASK) >> _DB_ANT_SIG_SHIFT
    def _set_db_ant_sig_present(self, val): self.present_flags = self.present_flags | (val << _DB_ANT_SIG_SHIFT)
    def _get_db_ant_noise_present(self): return (self.present_flags & _DB_ANT_NOISE_MASK) >> _DB_ANT_NOISE_SHIFT
    def _set_db_ant_noise_present(self, val): self.present_flags =    self.present_flags | (val << _DB_ANT_NOISE_SHIFT)
    def _get_rx_flags_present(self): return (self.present_flags & _RX_FLAGS_MASK) >> _RX_FLAGS_SHIFT
    def _set_rx_flags_present(self, val): self.present_flags = self.present_flags | (val << _RX_FLAGS_SHIFT)
    def _get_chanplus_present(self): return (self.present_flags & _CHANNELPLUS_MASK) >> _CHANNELPLUS_SHIFT
    def _set_chanplus_present(self, val): self.present_flags = self.present_flags | (val << _CHANNELPLUS_SHIFT)
    def _get_ext_present(self): return (self.present_flags & _EXT_MASK) >> _EXT_SHIFT
    def _set_ext_present(self, val): self.present_flags = self.present_flags | (val << _EXT_SHIFT)

    tsft_present = property(_get_tsft_present, _set_tsft_present)
    flags_present = property(_get_flags_present, _set_flags_present)
    rate_present = property(_get_rate_present, _set_rate_present)
    channel_present = property(_get_channel_present, _set_channel_present)
    fhss_present = property(_get_fhss_present, _set_fhss_present)
    ant_sig_present = property(_get_ant_sig_present, _set_ant_sig_present)
    ant_noise_present = property(_get_ant_noise_present, _set_ant_noise_present)
    lock_qual_present = property(_get_lock_qual_present, _set_lock_qual_present)
    tx_attn_present = property(_get_tx_attn_present, _set_tx_attn_present)
    db_tx_attn_present = property(_get_db_tx_attn_present, _set_db_tx_attn_present)
    dbm_tx_power_present = property(_get_dbm_power_present, _set_dbm_power_present)
    ant_present = property(_get_ant_present, _set_ant_present)
    db_ant_sig_present = property(_get_db_ant_sig_present, _set_db_ant_sig_present)
    db_ant_noise_present = property(_get_db_ant_noise_present, _set_db_ant_noise_present)
    rx_flags_present = property(_get_rx_flags_present, _set_rx_flags_present)
    chanplus_present = property(_get_chanplus_present, _set_chanplus_present)
    ext_present = property(_get_ext_present, _set_ext_present)

    def unpack(self, buf):
        dpkt.Packet.unpack(self, buf)
        self.data = buf[socket.ntohs(self.length):]

        self.fields = []
        buf = buf[self.__hdr_len__:]

        # decode each field into self.<name> (eg. self.tsft) as well as append it self.fields list
        field_decoder = [
            ('tsft', self.tsft_present, self.TSFT),
            ('flags', self.flags_present, self.Flags),
            ('rate', self.rate_present, self.Rate),
            ('channel', self.channel_present, self.Channel),
            ('fhss', self.fhss_present, self.FHSS),
            ('ant_sig', self.ant_sig_present, self.AntennaSignal),
            ('ant_noise', self.ant_noise_present, self.AntennaNoise),
            ('lock_qual', self.lock_qual_present, self.LockQuality),
            ('tx_attn', self.tx_attn_present, self.TxAttenuation),
            ('db_tx_attn', self.db_tx_attn_present, self.DbTxAttenuation),
            ('dbm_tx_power', self.dbm_tx_power_present, self.DbmTxPower),
            ('ant', self.ant_present, self.Antenna),
            ('db_ant_sig', self.db_ant_sig_present, self.DbAntennaSignal),
            ('db_ant_noise', self.db_ant_noise_present, self.DbAntennaNoise),
            ('rx_flags', self.rx_flags_present, self.RxFlags)
        ]
        for name, present_bit, parser in field_decoder:
            if present_bit:
                field = parser(buf)
                field.data = ''
                setattr(self, name, field)
                self.fields.append(field)
                buf = buf[len(field):]

        if len(self.data) > 0:
            if self.flags_present and self.flags.fcs:
                self.data = ieee80211.IEEE80211(self.data, fcs = self.flags.fcs)
            else:
                self.data = ieee80211.IEEE80211(self.data)

    class Antenna(dpkt.Packet):
        __hdr__ = (
            ('index', 'B',  0),
            )

    class AntennaNoise(dpkt.Packet):
        __hdr__ = (
            ('db', 'B', 0),
            )

    class AntennaSignal(dpkt.Packet):
        __hdr__ = (
            ('db',  'B', 0),
            )

    class Channel(dpkt.Packet):
        __hdr__ = (
            ('freq', 'H', 0),
            ('flags', 'H',  0),
            )

    class FHSS(dpkt.Packet):
        __hdr__ = (
            ('set', 'B', 0),
            ('pattern', 'B', 0),
            )

    class Flags(dpkt.Packet):
        __hdr__ = (
            ('val', 'B', 0),
            )

        def _get_fcs_present(self): return (self.val & _FCS_MASK) >> _FCS_SHIFT

        def _set_fcs_present(self, v): (v << _FCS_SHIFT) | (self.val & ~_FCS_MASK)
        fcs = property(_get_fcs_present, _set_fcs_present)

    class LockQuality(dpkt.Packet):
        __hdr__ = (
            ('val', 'H', 0),
            )

    class RxFlags(dpkt.Packet):
        __hdr__ = (
            ('val', 'H', 0),
            )

    class Rate(dpkt.Packet):
        __hdr__ = (
            ('val', 'B', 0),
            )

    class TSFT(dpkt.Packet):
        __hdr__ = (
            ('usecs', 'Q', 0),
            )

    class TxAttenuation(dpkt.Packet):
        __hdr__ = (
            ('val',  'H', 0),
            )

    class DbTxAttenuation(dpkt.Packet):
        __hdr__ = (
            ('db', 'H', 0),
            )

    class DbAntennaNoise(dpkt.Packet):
        __hdr__ = (
            ('db', 'B', 0),
            )

    class DbAntennaSignal(dpkt.Packet):
        __hdr__ = (
            ('db', 'B', 0),
            )

    class DbmTxPower(dpkt.Packet):
        __hdr__ = (
            ('dbm', 'B', 0),
            )

if __name__ == '__main__':
    import unittest

    class RadiotapTestCase(unittest.TestCase):
        def test_Radiotap(self):
            s = '\x00\x00\x00\x18\x6e\x48\x00\x00\x00\x02\x6c\x09\xa0\x00\xa8\x81\x02\x00\x00\x00\x00\x00\x00\x00'
            rad = Radiotap(s)
            self.failUnless(rad.version == 0)
            self.failUnless(rad.present_flags == 0x6e480000)
            self.failUnless(rad.tsft_present == 0)
            self.failUnless(rad.flags_present == 1)
            self.failUnless(rad.rate_present == 1)
            self.failUnless(rad.channel_present == 1)
            self.failUnless(rad.fhss_present == 0)
            self.failUnless(rad.ant_sig_present == 1)
            self.failUnless(rad.ant_noise_present == 1)
            self.failUnless(rad.lock_qual_present == 0)
            self.failUnless(rad.db_tx_attn_present == 0)
            self.failUnless(rad.dbm_tx_power_present == 0)
            self.failUnless(rad.ant_present == 1)
            self.failUnless(rad.db_ant_sig_present == 0)
            self.failUnless(rad.db_ant_noise_present == 0)
            self.failUnless(rad.rx_flags_present == 1)
            self.failUnless(rad.channel.freq == 0x6c09)
            self.failUnless(rad.channel.flags == 0xa000)
            self.failUnless(len(rad.fields) == 7)

        def test_fcs(self):
            s = '\x00\x00\x1a\x00\x2f\x48\x00\x00\x34\x8f\x71\x09\x00\x00\x00\x00\x10\x0c\x85\x09\xc0\x00\xcc\x01\x00\x00'
            rt = Radiotap(s)
            self.failUnless(rt.flags_present == 1)
            self.failUnless(rt.flags.fcs == 1)

    unittest.main()



#!/usr/bin/env python

import math, optparse, random, socket, sys, time
import dpkt

class Ping(object):
    def __init__(self):
        usage = '%prog [OPTIONS] <host>'
        self.op = optparse.OptionParser(usage=usage)
        self.op.add_option('-c', dest='count', type='int', default=sys.maxint,
                           help='Total number of queries to send')
        self.op.add_option('-i', dest='wait', type='float', default=1,
                           help='Specify packet interval timeout in seconds')

    def gen_ping(self, opts):
        pass
    def open_sock(self, opts):
        pass
    def print_header(self, opts):
        pass
    def print_reply(self, opts, buf):
        pass
    
    def main(self, argv=None):
        if not argv:
            argv = sys.argv[1:]
        opts, args = self.op.parse_args(argv)
    
        if not args:
            self.op.error('missing host')
        elif len(args) > 1:
            self.op.error('only one host may be specified')

        host = args[0]
        opts.ip = socket.gethostbyname(host)
        sock = self.open_sock(opts)
        
        sent = rcvd = rtt_max = rtt_sum = rtt_sumsq = 0
        rtt_min = 0xffff
        try:
            self.print_header(opts)
            for ping in self.gen_ping(opts):
                try:
                    start = time.time()
                    sock.send(ping)
                    buf = sock.recv(0xffff)
                    rtt = time.time() - start
                    
                    if rtt < rtt_min: rtt_min = rtt
                    if rtt > rtt_max: rtt_max = rtt
                    rtt_sum += rtt
                    rtt_sumsq += rtt * rtt

                    self.print_reply(opts, buf, rtt)
                    rcvd += 1
                except socket.timeout:
                    pass
                sent += 1
                time.sleep(opts.wait)
        except KeyboardInterrupt:
            pass

        print '\n--- %s ping statistics ---' % opts.ip
        print '%d packets transmitted, %d packets received, %.1f%% packet loss' % \
              (sent, rcvd, (float(sent - rcvd) / sent) * 100)
        rtt_avg = rtt_sum / sent
        if rtt_min == 0xffff: rtt_min = 0
        print 'round-trip min/avg/max/std-dev = %.3f/%.3f/%.3f/%.3f ms' % \
              (rtt_min * 1000, rtt_avg * 1000, rtt_max * 1000,
               math.sqrt((rtt_sumsq / sent) - (rtt_avg * rtt_avg)) * 1000)

class ICMPPing(Ping):
    def __init__(self):
        Ping.__init__(self)
        self.op.add_option('-p', dest='payload', type='string',
                           default='hello world!',
                           help='Echo payload string')

    def open_sock(self, opts):
        sock = socket.socket(socket.AF_INET, socket.SOCK_RAW, 1)
        sock.connect((opts.ip, 1))
        sock.settimeout(opts.wait)
        return sock
    
    def gen_ping(self, opts):
        for i in xrange(opts.count):
            icmp = dpkt.icmp.ICMP(
                type=8, data=dpkt.icmp.ICMP.Echo(id=random.randint(0, 0xffff),
                                                 seq=i, data=opts.payload))
            yield str(icmp)

    def print_header(self, opts):
        print 'PING %s: %d data bytes' % (opts.ip, len(opts.payload))
        
    def print_reply(self, opts, buf, rtt):
        ip = dpkt.ip.IP(buf)
        if sys.platform == 'darwin':
            # XXX - work around raw socket bug on MacOS X
            ip.data = ip.icmp = dpkt.icmp.ICMP(buf[20:])
            ip.len = len(ip.data)
        print '%d bytes from %s: icmp_seq=%d ip_id=%d ttl=%d time=%.3f ms' % \
              (len(ip.icmp), opts.ip, ip.icmp.echo.seq, ip.id, ip.ttl,
               rtt * 1000)

if __name__ == '__main__':
    p = ICMPPing()
    p.main()



# $Id: igmp.py 23 2006-11-08 15:45:33Z dugsong $

"""Internet Group Management Protocol."""

import dpkt

class IGMP(dpkt.Packet):
    __hdr__ = (
        ('type', 'B', 0),
        ('maxresp', 'B', 0),
        ('sum', 'H', 0),
        ('group', 'I', 0)
        )
    def __str__(self):
        if not self.sum:
            self.sum = dpkt.in_cksum(dpkt.Packet.__str__(self))
        return dpkt.Packet.__str__(self)



# $Id: 80211.py 53 2008-12-18 01:22:57Z jon.oberheide $

"""IEEE 802.11."""

import dpkt, socket, struct

# Frame Types
MGMT_TYPE           = 0
CTL_TYPE            = 1
DATA_TYPE           = 2

# Frame Sub-Types
M_ASSOC_REQ         = 0
M_ASSOC_RESP        = 1
M_REASSOC_REQ       = 2
M_REASSOC_RESP      = 3
M_PROBE_REQ         = 4
M_PROBE_RESP        = 5
M_BEACON            = 8
M_ATIM              = 9
M_DISASSOC          = 10
M_AUTH              = 11
M_DEAUTH            = 12
M_ACTION            = 13
C_BLOCK_ACK_REQ     = 8
C_BLOCK_ACK         = 9
C_PS_POLL           = 10
C_RTS               = 11
C_CTS               = 12
C_ACK               = 13
C_CF_END            = 14
C_CF_END_ACK        = 15
D_DATA              = 0
D_DATA_CF_ACK       = 1
D_DATA_CF_POLL      = 2
D_DATA_CF_ACK_POLL  = 3
D_NULL              = 4
D_CF_ACK            = 5
D_CF_POLL           = 6
D_CF_ACK_POLL       = 7
D_QOS_DATA          = 8
D_QOS_CF_ACK        = 9
D_QOS_CF_POLL       = 10
D_QOS_CF_ACK_POLL   = 11
D_QOS_NULL          = 12
D_QOS_CF_POLL_EMPTY = 14

TO_DS_FLAG          = 10
FROM_DS_FLAG        = 1
INTER_DS_FLAG       = 11

# Bitshifts for Frame Control
_VERSION_MASK       = 0x0300
_TYPE_MASK          = 0x0c00
_SUBTYPE_MASK       = 0xf000
_TO_DS_MASK         = 0x0001
_FROM_DS_MASK       = 0x0002
_MORE_FRAG_MASK     = 0x0004
_RETRY_MASK         = 0x0008
_PWR_MGT_MASK       = 0x0010
_MORE_DATA_MASK     = 0x0020
_WEP_MASK           = 0x0040
_ORDER_MASK         = 0x0080
_VERSION_SHIFT      = 8
_TYPE_SHIFT         = 10
_SUBTYPE_SHIFT      = 12
_TO_DS_SHIFT        = 0
_FROM_DS_SHIFT      = 1
_MORE_FRAG_SHIFT    = 2
_RETRY_SHIFT        = 3
_PWR_MGT_SHIFT      = 4
_MORE_DATA_SHIFT    = 5
_WEP_SHIFT          = 6
_ORDER_SHIFT        = 7

# IEs
IE_SSID    = 0
IE_RATES   = 1
IE_FH      = 2
IE_DS      = 3
IE_CF      = 4
IE_TIM     = 5
IE_IBSS    = 6
IE_HT_CAPA = 45
IE_ESR     = 50
IE_HT_INFO = 61

FCS_LENGTH = 4

FRAMES_WITH_CAPABILITY = [ M_BEACON,
                           M_ASSOC_RESP,
                           M_ASSOC_REQ,
                           M_REASSOC_REQ,
                         ]

# Block Ack control constants
_ACK_POLICY_SHIFT = 0
_MULTI_TID_SHIFT  = 1
_COMPRESSED_SHIFT = 2
_TID_SHIFT        = 12

_ACK_POLICY_MASK = 0x0001
_MULTI_TID_MASK  = 0x0002
_COMPRESSED_MASK = 0x0004
_TID_MASK        = 0xf000

_COMPRESSED_BMP_LENGTH = 8
_BMP_LENGTH = 128

# Action frame categories
BLOCK_ACK = 3

# Block ack category action codes
BLOCK_ACK_CODE_REQUEST = 0
BLOCK_ACK_CODE_RESPONSE = 1

class IEEE80211(dpkt.Packet):
    __hdr__ = (
        ('framectl', 'H', 0),
        ('duration', 'H', 0)
        )

    def _get_version(self): return (self.framectl & _VERSION_MASK) >> _VERSION_SHIFT
    def _set_version(self, val): self.framectl = (val << _VERSION_SHIFT) | (self.framectl & ~_VERSION_MASK)
    def _get_type(self): return (self.framectl & _TYPE_MASK) >> _TYPE_SHIFT
    def _set_type(self, val): self.framectl = (val << _TYPE_SHIFT) | (self.framectl & ~_TYPE_MASK)
    def _get_subtype(self): return (self.framectl & _SUBTYPE_MASK) >> _SUBTYPE_SHIFT
    def _set_subtype(self, val): self.framectl = (val << _SUBTYPE_SHIFT) | (self.framectl & ~_SUBTYPE_MASK)
    def _get_to_ds(self): return (self.framectl & _TO_DS_MASK) >> _TO_DS_SHIFT
    def _set_to_ds(self, val): self.framectl = (val << _TO_DS_SHIFT) | (self.framectl & ~_TO_DS_MASK)
    def _get_from_ds(self): return (self.framectl & _FROM_DS_MASK) >> _FROM_DS_SHIFT
    def _set_from_ds(self, val): self.framectl = (val << _FROM_DS_SHIFT) | (self.framectl & ~_FROM_DS_MASK)
    def _get_more_frag(self): return (self.framectl & _MORE_FRAG_MASK) >> _MORE_FRAG_SHIFT
    def _set_more_frag(self, val): self.framectl = (val << _MORE_FRAG_SHIFT) | (self.framectl & ~_MORE_FRAG_MASK)
    def _get_retry(self): return (self.framectl & _RETRY_MASK) >> _RETRY_SHIFT
    def _set_retry(self, val): self.framectl = (val << _RETRY_SHIFT) | (self.framectl & ~_RETRY_MASK)
    def _get_pwr_mgt(self): return (self.framectl & _PWR_MGT_MASK) >> _PWR_MGT_SHIFT
    def _set_pwr_mgt(self, val): self.framectl = (val << _PWR_MGT_SHIFT) | (self.framectl & ~_PWR_MGT_MASK)
    def _get_more_data(self): return (self.framectl & _MORE_DATA_MASK) >> _MORE_DATA_SHIFT
    def _set_more_data(self, val): self.framectl = (val << _MORE_DATA_SHIFT) | (self.framectl & ~_MORE_DATA_MASK)
    def _get_wep(self): return (self.framectl & _WEP_MASK) >> _WEP_SHIFT
    def _set_wep(self, val): self.framectl = (val << _WEP_SHIFT) | (self.framectl & ~_WEP_MASK)
    def _get_order(self): return (self.framectl & _ORDER_MASK) >> _ORDER_SHIFT
    def _set_order(self, val): self.framectl = (val << _ORDER_SHIFT) | (self.framectl & ~_ORDER_MASK)

    version = property(_get_version, _set_version)
    type = property(_get_type, _set_type)
    subtype = property(_get_subtype, _set_subtype)
    to_ds = property(_get_to_ds, _set_to_ds)
    from_ds = property(_get_from_ds, _set_from_ds)
    more_frag = property(_get_more_frag, _set_more_frag)
    retry = property(_get_retry, _set_retry)
    pwr_mgt = property(_get_pwr_mgt, _set_pwr_mgt)
    more_data = property(_get_more_data, _set_more_data)
    wep = property(_get_wep, _set_wep)
    order = property(_get_order, _set_order)

    def unpack_ies(self, buf):
        self.ies = []

        ie_decoder = {
           IE_SSID:     ('ssid',    self.IE),
           IE_RATES:    ('rate',    self.IE),
           IE_FH:       ('fh',      self.FH),
           IE_DS:       ('ds',      self.DS),
           IE_CF:       ('cf',      self.CF),
           IE_TIM:      ('tim',     self.TIM),
           IE_IBSS:     ('ibss',    self.IBSS),
           IE_HT_CAPA:  ('ht_capa', self.IE),
           IE_ESR:      ('esr',     self.IE),
           IE_HT_INFO:  ('ht_info', self.IE)
        }

        # each IE starts with an ID and a length
        while len(buf) > FCS_LENGTH:
            ie_id = struct.unpack('B',(buf[0]))[0]
            try:
                parser = ie_decoder[ie_id][1]
                name = ie_decoder[ie_id][0]
            except KeyError:
                parser = self.IE
                name = 'ie_' + str(ie_id)
            ie = parser(buf)

            ie.data = buf[2:2+ie.len]
            setattr(self, name, ie)
            self.ies.append(ie)
            buf = buf[2+ie.len:]

    class Capability:
        def __init__(self, field):
            self.ess = field & 1
            self.ibss = (field >> 1) & 1
            self.cf_poll = (field >> 2) & 1
            self.cf_poll_req = (field >> 3) & 1
            self.privacy = (field >> 4) & 1
            self.short_preamble = (field >> 5) & 1
            self.pbcc = (field >> 6) & 1
            self.hopping = (field >> 7) & 1
            self.spec_mgmt = (field >> 8) & 1
            self.qos = (field >> 9) & 1
            self.short_slot = (field >> 10) & 1
            self.apsd = (field >> 11) & 1
            self.dsss = (field >> 13) & 1
            self.delayed_blk_ack = (field >> 14) & 1
            self.imm_blk_ack = (field >> 15) & 1

    def __init__(self, *args, **kwargs):
        if kwargs and 'fcs' in kwargs:
            self.fcs_present = kwargs.pop('fcs')
        else:
            self.fcs_present = False

        super(IEEE80211, self).__init__(*args, **kwargs)

    def unpack(self, buf):
        dpkt.Packet.unpack(self, buf)
        self.data = buf[self.__hdr_len__:]

        m_decoder = {
            M_BEACON:       ('beacon',      self.Beacon),
            M_ASSOC_REQ:    ('assoc_req',   self.Assoc_Req),
            M_ASSOC_RESP:   ('assoc_resp',  self.Assoc_Resp),
            M_DISASSOC:     ('diassoc',     self.Disassoc),
            M_REASSOC_REQ:  ('reassoc_req', self.Reassoc_Req),
            M_REASSOC_RESP: ('reassoc_resp',self.Assoc_Resp),
            M_AUTH:         ('auth',        self.Auth),
            M_PROBE_RESP:   ('probe_resp',  self.Beacon),
            M_DEAUTH:       ('deauth',      self.Deauth),
            M_ACTION:       ('action',      self.Action)
        }

        c_decoder = {
            C_RTS:          ('rts',         self.RTS),
            C_CTS:          ('cts',         self.CTS),
            C_ACK:          ('ack',         self.ACK),
            C_BLOCK_ACK_REQ:('bar',         self.BlockAckReq),
            C_BLOCK_ACK:    ('back',        self.BlockAck),
            C_CF_END:       ('cf_end',      self.CFEnd),
        }

        d_dsData = {
            0               :   self.Data,
            FROM_DS_FLAG    :   self.DataFromDS,
            TO_DS_FLAG      :   self.DataToDS,
            INTER_DS_FLAG   :   self.DataInterDS
        }


        # For now decode everything with DATA. Haven't checked about other QoS
        # additions
        d_decoder = {
            # modified the decoder to consider the ToDS and FromDS flags
            # Omitting the 11 case for now
            D_DATA:         ('data_frame',  d_dsData),
            D_NULL:         ('data_frame',  d_dsData),
            D_QOS_DATA:     ('data_frame',  d_dsData),
            D_QOS_NULL:     ('data_frame',  d_dsData)
        }

        decoder = {
            MGMT_TYPE:m_decoder,
            CTL_TYPE:c_decoder,
            DATA_TYPE:d_decoder
        }

        # Strip off the FCS field
        if self.fcs_present:
            self.fcs = struct.unpack('I', self.data[-1 * FCS_LENGTH:])[0]
            self.data = self.data[0: -1 * FCS_LENGTH]

        if self.type == MGMT_TYPE:
            self.mgmt = self.MGMT_Frame(self.data)
            self.data = self.mgmt.data
            if self.subtype == M_PROBE_REQ:
                self.unpack_ies(self.data)
                return
            if self.subtype == M_ATIM:
                return

        try:
            parser = decoder[self.type][self.subtype][1]
            name = decoder[self.type][self.subtype][0]
        except KeyError:
            print "Key error:", self.type, self.subtype
            return

        if self.type == DATA_TYPE:
            # need to grab the ToDS/FromDS info
            parser = parser[self.to_ds*10+self.from_ds]

        if self.type == MGMT_TYPE:
            field = parser(self.mgmt.data)
        else:
            field = parser(self.data)
            self.data = field

        setattr(self, name, field)

        if self.type == MGMT_TYPE:
            self.ies = self.unpack_ies(field.data)
            if self.subtype in FRAMES_WITH_CAPABILITY:
                self.capability = self.Capability(socket.ntohs(field.capability))

        if self.type == DATA_TYPE and self.subtype == D_QOS_DATA:
            self.qos_data = self.QoS_Data(field.data)
            field.data = self.qos_data.data

        self.data = field.data

    class BlockAckReq(dpkt.Packet):
        __hdr__ = (
            ('dst', '6s', '\x00' * 6),
            ('src', '6s', '\x00' *6),
            ('ctl', 'H', 0),
            ('seq', 'H', 0),
            )

    class BlockAck(dpkt.Packet):
        __hdr__ = (
            ('dst', '6s', '\x00' * 6),
            ('src', '6s', '\x00' * 6),
            ('ctl', 'H', 0),
            ('seq', 'H', 0),
            )

        def _get_compressed(self): return (self.ctl & _COMPRESSED_MASK) >> _COMPRESSED_SHIFT
        def _set_compressed(self, val): self.ctl = (val << _COMPRESSED_SHIFT) | (self.ctl & ~_COMPRESSED_MASK)

        def _get_ack_policy(self): return (self.ctl & _ACK_POLICY_MASK) >> _ACK_POLICY_SHIFT
        def _set_ack_policy(self, val): self.ctl = (val << _ACK_POLICY_SHIFT) | (self.ctl & ~_ACK_POLICY_MASK)

        def _get_multi_tid(self): return (self.ctl & _MULTI_TID_MASK) >> _MULTI_TID_SHIFT
        def _set_multi_tid(self, val): self.ctl = (val << _MULTI_TID_SHIFT) | (self.ctl & ~_MULTI_TID_MASK)

        def _get_tid(self): return (self.ctl & _TID_MASK) >> _TID_SHIFT
        def _set_tid(self, val): self.ctl = (val << _TID_SHIFT) | (self.ctl & ~_TID_MASK)

        compressed = property(_get_compressed, _set_compressed)
        ack_policy = property(_get_ack_policy, _set_ack_policy)
        multi_tid = property(_get_multi_tid, _set_multi_tid)
        tid = property(_get_tid, _set_tid)

        def unpack(self, buf):
            dpkt.Packet.unpack(self, buf)
            self.data = buf[self.__hdr_len__:]
            self.ctl = socket.ntohs(self.ctl)

            if self.compressed:
                self.bmp = struct.unpack('8s', self.data[0:_COMPRESSED_BMP_LENGTH])[0]
            else:
                self.bmp = struct.unpack('128s', self.data[0:_BMP_LENGTH])[0]
            self.data = self.data[len(self.__hdr__) + len(self.bmp):]

    class RTS(dpkt.Packet):
        __hdr__ = (
            ('dst', '6s', '\x00' * 6),
            ('src', '6s', '\x00' * 6)
            )

    class CTS(dpkt.Packet):
        __hdr__ = (
            ('dst', '6s', '\x00' * 6),
            )

    class ACK(dpkt.Packet):
        __hdr__ = (
            ('dst', '6s', '\x00' * 6),
            )

    class CFEnd(dpkt.Packet):
        __hdr__ = (
            ('dst', '6s', '\x00' *6),
            ('src', '6s', '\x00' *6),
            )

    class MGMT_Frame(dpkt.Packet):
        __hdr__ = (
            ('dst', '6s', '\x00' *6),
            ('src', '6s', '\x00' *6),
            ('bssid', '6s', '\x00' *6),
            ('frag_seq', 'H', 0)
            )

    class Beacon(dpkt.Packet):
        __hdr__ = (
            ('timestamp', 'Q', 0),
            ('interval', 'H', 0),
            ('capability', 'H', 0)
            )

    class Disassoc(dpkt.Packet):
        __hdr__ = (
            ('reason', 'H', 0),
            )

    class Assoc_Req(dpkt.Packet):
        __hdr__ = (
            ('capability', 'H', 0),
            ('interval', 'H', 0)
            )

    class Assoc_Resp(dpkt.Packet):
        __hdr__ = (
            ('capability', 'H', 0),
            ('status', 'H', 0),
            ('aid', 'H', 0)
            )

    class Reassoc_Req(dpkt.Packet):
        __hdr__ = (
            ('capability', 'H', 0),
            ('interval', 'H', 0),
            ('current_ap', '6s', '\x00'*6)
            )

    # This obviously doesn't support any of AUTH frames that use encryption
    class Auth(dpkt.Packet):
        __hdr__ = (
            ('algorithm', 'H', 0),
            ('auth_seq', 'H', 0),
            )

    class Deauth(dpkt.Packet):
        __hdr__ = (
            ('reason', 'H', 0),
            )

    class Action(dpkt.Packet):
        __hdr__ = (
            ('category', 'B', 0),
            ('code', 'B', 0),
            )

        def unpack(self, buf):
            dpkt.Packet.unpack(self, buf)

            action_parser = {
                BLOCK_ACK: { BLOCK_ACK_CODE_REQUEST: ('block_ack_request', IEEE80211.BlockAckActionRequest),
                              BLOCK_ACK_CODE_RESPONSE: ('block_ack_response', IEEE80211.BlockAckActionResponse),
                            },
            }

            decoder = action_parser[self.category][self.code][1]
            field_name = action_parser[self.category][self.code][0]
            field = decoder(self.data)
            setattr(self, field_name, field)
            self.data = field.data

    class BlockAckActionRequest(dpkt.Packet):
        __hdr__ = (
            ('dialog', 'B', 0),
            ('parameters', 'H', 0),
            ('timeout', 'H', 0),
            ('starting_seq', 'H', 0),
            )

    class BlockAckActionResponse(dpkt.Packet):
        __hdr__ = (
            ('dialog', 'B', 0),
            ('status_code', 'H', 0),
            ('parameters', 'H', 0),
            ('timeout', 'H', 0),
            )

    class Data(dpkt.Packet):
        __hdr__ = (
            ('dst', '6s', '\x00'*6),
            ('src', '6s', '\x00'*6),
            ('bssid', '6s', '\x00'*6),
            ('frag_seq', 'H', 0)
            )


    class DataFromDS(dpkt.Packet):
        __hdr__ = (
            ('dst', '6s', '\x00'*6),
            ('bssid', '6s', '\x00'*6),
            ('src', '6s', '\x00'*6),
            ('frag_seq', 'H', 0)
            )


    class DataToDS(dpkt.Packet):
        __hdr__ = (
            ('bssid', '6s', '\x00'*6),
            ('src', '6s', '\x00'*6),
            ('dst', '6s', '\x00'*6),
            ('frag_seq', 'H', 0)
            )

    class DataInterDS(dpkt.Packet):
        __hdr__ = (
            ('dst', '6s', '\x00'*6),
            ('src', '6s', '\x00'*6),
            ('da', '6s', '\x00'*6),
            ('frag_seq', 'H', 0),
            ('sa', '6s', '\x00'*6)
            )

    class QoS_Data(dpkt.Packet):
        __hdr__ = (
            ('control', 'H', 0),
            )

    class IE(dpkt.Packet):
        __hdr__ = (
            ('id', 'B', 0),
            ('len', 'B', 0)
            )
        def unpack(self, buf):
            dpkt.Packet.unpack(self, buf)
            self.info = buf[2:self.len+ 2]

    class FH(dpkt.Packet):
        __hdr__ = (
            ('id', 'B', 0),
            ('len', 'B', 0),
            ('tu', 'H', 0),
            ('hopset', 'B', 0),
            ('hoppattern', 'B', 0),
            ('hopindex', 'B', 0)
            )

    class DS(dpkt.Packet):
        __hdr__ = (
            ('id', 'B', 0),
            ('len', 'B', 0),
            ('ch', 'B', 0)
            )

    class CF(dpkt.Packet):
        __hdr__ = (
            ('id', 'B', 0),
            ('len', 'B', 0),
            ('count', 'B', 0),
            ('period', 'B', 0),
            ('max', 'H', 0),
            ('dur', 'H', 0)
            )

    class TIM(dpkt.Packet):
       __hdr__ = (
            ('id', 'B', 0),
            ('len', 'B', 0),
            ('count', 'B', 0),
            ('period', 'B', 0),
            ('ctrl', 'H', 0)
            )
       def unpack(self, buf):
            dpkt.Packet.unpack(self, buf)
            self.bitmap = buf[5:self.len+ 2]

    class IBSS(dpkt.Packet):
       __hdr__ = (
            ('id', 'B', 0),
            ('len', 'B', 0),
            ('atim', 'H', 0)
            )



if __name__ == '__main__':
    import unittest

    class IEEE80211TestCase(unittest.TestCase):
        def test_802211_ack(self):
            s = '\xd4\x00\x00\x00\x00\x12\xf0\xb6\x1c\xa4\xff\xff\xff\xff'
            ieee = IEEE80211(s, fcs = True)
            self.failUnless(ieee.version == 0)
            self.failUnless(ieee.type == CTL_TYPE)
            self.failUnless(ieee.subtype == C_ACK)
            self.failUnless(ieee.to_ds == 0)
            self.failUnless(ieee.from_ds == 0)
            self.failUnless(ieee.pwr_mgt == 0)
            self.failUnless(ieee.more_data == 0)
            self.failUnless(ieee.wep == 0)
            self.failUnless(ieee.order == 0)
            self.failUnless(ieee.ack.dst == '\x00\x12\xf0\xb6\x1c\xa4')
            fcs = struct.unpack('I', s[-4:])[0]
            self.failUnless(ieee.fcs == fcs)

        def test_80211_beacon(self):
            s = '\x80\x00\x00\x00\xff\xff\xff\xff\xff\xff\x00\x26\xcb\x18\x6a\x30\x00\x26\xcb\x18\x6a\x30\xa0\xd0\x77\x09\x32\x03\x8f\x00\x00\x00\x66\x00\x31\x04\x00\x04\x43\x41\x45\x4e\x01\x08\x82\x84\x8b\x0c\x12\x96\x18\x24\x03\x01\x01\x05\x04\x00\x01\x00\x00\x07\x06\x55\x53\x20\x01\x0b\x1a\x0b\x05\x00\x00\x6e\x00\x00\x2a\x01\x02\x2d\x1a\x6e\x18\x1b\xff\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x30\x14\x01\x00\x00\x0f\xac\x04\x01\x00\x00\x0f\xac\x04\x01\x00\x00\x0f\xac\x01\x28\x00\x32\x04\x30\x48\x60\x6c\x36\x03\x51\x63\x03\x3d\x16\x01\x00\x05\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x85\x1e\x05\x00\x8f\x00\x0f\x00\xff\x03\x59\x00\x63\x73\x65\x2d\x33\x39\x31\x32\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x36\x96\x06\x00\x40\x96\x00\x14\x00\xdd\x18\x00\x50\xf2\x02\x01\x01\x80\x00\x03\xa4\x00\x00\x27\xa4\x00\x00\x42\x43\x5e\x00\x62\x32\x2f\x00\xdd\x06\x00\x40\x96\x01\x01\x04\xdd\x05\x00\x40\x96\x03\x05\xdd\x05\x00\x40\x96\x0b\x09\xdd\x08\x00\x40\x96\x13\x01\x00\x34\x01\xdd\x05\x00\x40\x96\x14\x05'
            ieee = IEEE80211(s, fcs = True)
            self.failUnless(ieee.version == 0)
            self.failUnless(ieee.type == MGMT_TYPE)
            self.failUnless(ieee.subtype == M_BEACON)
            self.failUnless(ieee.to_ds == 0)
            self.failUnless(ieee.from_ds == 0)
            self.failUnless(ieee.pwr_mgt == 0)
            self.failUnless(ieee.more_data == 0)
            self.failUnless(ieee.wep == 0)
            self.failUnless(ieee.order == 0)
            self.failUnless(ieee.mgmt.dst == '\xff\xff\xff\xff\xff\xff')
            self.failUnless(ieee.mgmt.src == '\x00\x26\xcb\x18\x6a\x30')
            self.failUnless(ieee.beacon.capability == 0x3104)
            self.failUnless(ieee.capability.privacy == 1)
            self.failUnless(ieee.ssid.data == 'CAEN')
            self.failUnless(ieee.rate.data == '\x82\x84\x8b\x0c\x12\x96\x18\x24')
            self.failUnless(ieee.ds.data == '\x01')
            self.failUnless(ieee.tim.data == '\x00\x01\x00\x00')
            fcs = struct.unpack('I', s[-4:])[0]
            self.failUnless(ieee.fcs == fcs)

        def test_80211_data(self):
            s = '\x08\x09\x20\x00\x00\x26\xcb\x17\x3d\x91\x00\x16\x44\xb0\xae\xc6\x00\x02\xb3\xd6\x26\x3c\x80\x7e\xaa\xaa\x03\x00\x00\x00\x08\x00\x45\x00\x00\x28\x07\x27\x40\x00\x80\x06\x1d\x39\x8d\xd4\x37\x3d\x3f\xf5\xd1\x69\xc0\x5f\x01\xbb\xb2\xd6\xef\x23\x38\x2b\x4f\x08\x50\x10\x42\x04\xac\x17\x00\x00'
            ieee = IEEE80211(s, fcs = True)
            self.failUnless(ieee.type == DATA_TYPE)
            self.failUnless(ieee.subtype == D_DATA)
            self.failUnless(ieee.data_frame.dst == '\x00\x02\xb3\xd6\x26\x3c')
            self.failUnless(ieee.data_frame.src == '\x00\x16\x44\xb0\xae\xc6')
            self.failUnless(ieee.data_frame.frag_seq == 0x807e)
            self.failUnless(ieee.data == '\xaa\xaa\x03\x00\x00\x00\x08\x00\x45\x00\x00\x28\x07\x27\x40\x00\x80\x06\x1d\x39\x8d\xd4\x37\x3d\x3f\xf5\xd1\x69\xc0\x5f\x01\xbb\xb2\xd6\xef\x23\x38\x2b\x4f\x08\x50\x10\x42\x04')
            self.failUnless(ieee.fcs == struct.unpack('I', '\xac\x17\x00\x00')[0])

            import llc, ip
            llc_pkt = llc.LLC(ieee.data_frame.data)
            ip_pkt = ip.IP(llc_pkt.data)
            self.failUnless(ip_pkt.dst == '\x3f\xf5\xd1\x69')

        def test_80211_data_qos(self):
            s = '\x88\x01\x3a\x01\x00\x26\xcb\x17\x44\xf0\x00\x23\xdf\xc9\xc0\x93\x00\x26\xcb\x17\x44\xf0\x20\x7b\x00\x00\xaa\xaa\x03\x00\x00\x00\x88\x8e\x01\x00\x00\x74\x02\x02\x00\x74\x19\x80\x00\x00\x00\x6a\x16\x03\x01\x00\x65\x01\x00\x00\x61\x03\x01\x4b\x4c\xa7\x7e\x27\x61\x6f\x02\x7b\x3c\x72\x39\xe3\x7b\xd7\x43\x59\x91\x7f\xaa\x22\x47\x51\xb6\x88\x9f\x85\x90\x87\x5a\xd1\x13\x20\xe0\x07\x00\x00\x68\xbd\xa4\x13\xb0\xd5\x82\x7e\xc7\xfb\xe7\xcc\xab\x6e\x5d\x5a\x51\x50\xd4\x45\xc5\xa1\x65\x53\xad\xb5\x88\x5b\x00\x1a\x00\x2f\x00\x05\x00\x04\x00\x35\x00\x0a\x00\x09\x00\x03\x00\x08\x00\x33\x00\x39\x00\x16\x00\x15\x00\x14\x01\x00\xff\xff\xff\xff'
            ieee = IEEE80211(s, fcs = True)
            self.failUnless(ieee.type == DATA_TYPE)
            self.failUnless(ieee.subtype == D_QOS_DATA)
            self.failUnless(ieee.data_frame.dst == '\x00\x26\xcb\x17\x44\xf0')
            self.failUnless(ieee.data_frame.src == '\x00\x23\xdf\xc9\xc0\x93')
            self.failUnless(ieee.data_frame.frag_seq == 0x207b)
            self.failUnless(ieee.data == '\xaa\xaa\x03\x00\x00\x00\x88\x8e\x01\x00\x00\x74\x02\x02\x00\x74\x19\x80\x00\x00\x00\x6a\x16\x03\x01\x00\x65\x01\x00\x00\x61\x03\x01\x4b\x4c\xa7\x7e\x27\x61\x6f\x02\x7b\x3c\x72\x39\xe3\x7b\xd7\x43\x59\x91\x7f\xaa\x22\x47\x51\xb6\x88\x9f\x85\x90\x87\x5a\xd1\x13\x20\xe0\x07\x00\x00\x68\xbd\xa4\x13\xb0\xd5\x82\x7e\xc7\xfb\xe7\xcc\xab\x6e\x5d\x5a\x51\x50\xd4\x45\xc5\xa1\x65\x53\xad\xb5\x88\x5b\x00\x1a\x00\x2f\x00\x05\x00\x04\x00\x35\x00\x0a\x00\x09\x00\x03\x00\x08\x00\x33\x00\x39\x00\x16\x00\x15\x00\x14\x01\x00')
            self.failUnless(ieee.qos_data.control == 0x0)
            self.failUnless(ieee.fcs == struct.unpack('I', '\xff\xff\xff\xff')[0])

        def test_bug(self):
            s = '\x88\x41\x2c\x00\x00\x26\xcb\x17\x44\xf0\x00\x1e\x52\x97\x14\x11\x00\x1f\x6d\xe8\x18\x00\xd0\x07\x00\x00\x6f\x00\x00\x20\x00\x00\x00\x00'
            ieee = IEEE80211(s)
            self.failUnless(ieee.wep == 1)

        def test_data_ds(self):
            # verifying the ToDS and FromDS fields and that we're getting the
            # correct values

            s = '\x08\x03\x00\x00\x01\x0b\x85\x00\x00\x00\x00\x26\xcb\x18\x73\x50\x01\x0b\x85\x00\x00\x00\x00\x89\x00\x26\xcb\x18\x73\x50'
            ieee = IEEE80211(s)
            self.failUnless(ieee.type == DATA_TYPE)
            self.failUnless(ieee.to_ds == 1)
            self.failUnless(ieee.from_ds == 1)
            self.failUnless(ieee.data_frame.sa == '\x00\x26\xcb\x18\x73\x50')
            self.failUnless(ieee.data_frame.src == '\x00\x26\xcb\x18\x73\x50')
            self.failUnless(ieee.data_frame.dst == '\x01\x0b\x85\x00\x00\x00')
            self.failUnless(ieee.data_frame.da == '\x01\x0b\x85\x00\x00\x00')

            s = '\x88\x41\x50\x01\x00\x26\xcb\x17\x48\xc1\x00\x24\x2c\xe7\xfe\x8a\xff\xff\xff\xff\xff\xff\x80\xa0\x00\x00\x09\x1a\x00\x20\x00\x00\x00\x00'
            ieee = IEEE80211(s)
            self.failUnless(ieee.type == DATA_TYPE)
            self.failUnless(ieee.to_ds == 1)
            self.failUnless(ieee.from_ds == 0)
            self.failUnless(ieee.data_frame.bssid == '\x00\x26\xcb\x17\x48\xc1')
            self.failUnless(ieee.data_frame.src == '\x00\x24\x2c\xe7\xfe\x8a')
            self.failUnless(ieee.data_frame.dst == '\xff\xff\xff\xff\xff\xff')

            s = '\x08\x02\x02\x01\x00\x02\x44\xac\x27\x70\x00\x1f\x33\x39\x75\x44\x00\x1f\x33\x39\x75\x44\x90\xa4'
            ieee = IEEE80211(s)
            self.failUnless(ieee.type == DATA_TYPE)
            self.failUnless(ieee.to_ds == 0)
            self.failUnless(ieee.from_ds == 1)
            self.failUnless(ieee.data_frame.bssid == '\x00\x1f\x33\x39\x75\x44')
            self.failUnless(ieee.data_frame.src == '\x00\x1f\x33\x39\x75\x44')
            self.failUnless(ieee.data_frame.dst == '\x00\x02\x44\xac\x27\x70')

        def test_compressed_block_ack(self):
            s = '\x94\x00\x00\x00\x34\xc0\x59\xd6\x3f\x62\xb4\x75\x0e\x46\x83\xc1\x05\x50\x80\xee\x03\x00\x00\x00\x00\x00\x00\x00\xa2\xe4\x98\x45'
            ieee = IEEE80211(s, fcs = True)
            self.failUnless(ieee.type == CTL_TYPE)
            self.failUnless(ieee.subtype == C_BLOCK_ACK)
            self.failUnless(ieee.back.dst == '\x34\xc0\x59\xd6\x3f\x62')
            self.failUnless(ieee.back.src == '\xb4\x75\x0e\x46\x83\xc1')
            self.failUnless(ieee.back.compressed == 1)
            self.failUnless(len(ieee.back.bmp) == 8)
            self.failUnless(ieee.back.ack_policy == 1)
            self.failUnless(ieee.back.tid == 5)

        def test_action_block_ack_request(self):
            s = '\xd0\x00\x3a\x01\x00\x23\x14\x36\x52\x30\xb4\x75\x0e\x46\x83\xc1\xb4\x75\x0e\x46\x83\xc1\x70\x14\x03\x00\x0d\x02\x10\x00\x00\x40\x29\x06\x50\x33\x9e'
            ieee = IEEE80211(s, fcs = True)
            self.failUnless(ieee.type == MGMT_TYPE)
            self.failUnless(ieee.subtype == M_ACTION)
            self.failUnless(ieee.action.category == BLOCK_ACK)
            self.failUnless(ieee.action.code == BLOCK_ACK_CODE_REQUEST)
            self.failUnless(ieee.action.block_ack_request.timeout == 0)
            parameters = struct.unpack('H', '\x10\x02')[0]
            self.failUnless(ieee.action.block_ack_request.parameters == parameters)

        def test_action_block_ack_response(self):
            s = '\xd0\x00\x3c\x00\xb4\x75\x0e\x46\x83\xc1\x00\x23\x14\x36\x52\x30\xb4\x75\x0e\x46\x83\xc1\xd0\x68\x03\x01\x0d\x00\x00\x02\x10\x88\x13\x9f\xc0\x0b\x75'
            ieee = IEEE80211(s, fcs = True)
            self.failUnless(ieee.type == MGMT_TYPE)
            self.failUnless(ieee.subtype == M_ACTION)
            self.failUnless(ieee.action.category == BLOCK_ACK)
            self.failUnless(ieee.action.code == BLOCK_ACK_CODE_RESPONSE)
            timeout = struct.unpack('H', '\x13\x88')[0]
            self.failUnless(ieee.action.block_ack_response.timeout == timeout)
            parameters = struct.unpack('H', '\x10\x02')[0]
            self.failUnless(ieee.action.block_ack_response.parameters == parameters)

    unittest.main()



# $Id: dns.py 27 2006-11-21 01:22:52Z dahelder $

"""Domain Name System."""

import struct
import dpkt

DNS_Q = 0
DNS_R = 1

# Opcodes
DNS_QUERY = 0
DNS_IQUERY = 1
DNS_STATUS = 2
DNS_NOTIFY = 4
DNS_UPDATE = 5

# Flags
DNS_CD = 0x0010	# checking disabled
DNS_AD = 0x0020	# authenticated data
DNS_Z =  0x0040	# unused
DNS_RA = 0x0080	# recursion available
DNS_RD = 0x0100	# recursion desired
DNS_TC = 0x0200	# truncated
DNS_AA = 0x0400	# authoritative answer
DNS_QR = 0x8000 # response ( query / response )

# Response codes
DNS_RCODE_NOERR = 0
DNS_RCODE_FORMERR = 1
DNS_RCODE_SERVFAIL = 2
DNS_RCODE_NXDOMAIN = 3
DNS_RCODE_NOTIMP = 4
DNS_RCODE_REFUSED = 5
DNS_RCODE_YXDOMAIN = 6
DNS_RCODE_YXRRSET = 7
DNS_RCODE_NXRRSET = 8
DNS_RCODE_NOTAUTH = 9
DNS_RCODE_NOTZONE = 10

# RR types
DNS_A = 1
DNS_NS = 2
DNS_CNAME = 5
DNS_SOA = 6
DNS_PTR = 12
DNS_HINFO = 13
DNS_MX = 15
DNS_TXT = 16
DNS_AAAA = 28
DNS_SRV = 33

# RR classes
DNS_IN = 1
DNS_CHAOS = 3
DNS_HESIOD = 4
DNS_ANY = 255

def pack_name(name, off, label_ptrs):
    if name:
        labels = name.split('.')
    else:
        labels = []
    labels.append('')
    buf = ''
    for i, label in enumerate(labels):
        key = '.'.join(labels[i:]).upper()
        ptr = label_ptrs.get(key)
        if not ptr:
            if len(key) > 1:
                ptr = off + len(buf)
                if ptr < 0xc000:
                    label_ptrs[key] = ptr
            i = len(label)
            buf += chr(i) + label
        else:
            buf += struct.pack('>H', (0xc000 | ptr))
            break
    return buf

def unpack_name(buf, off):
    name = ''
    saved_off = 0
    for i in range(100): # XXX
        n = ord(buf[off])
        if n == 0:
            off += 1
            break
        elif (n & 0xc0) == 0xc0:
            ptr = struct.unpack('>H', buf[off:off+2])[0] & 0x3fff
            off += 2
            if not saved_off:
                saved_off = off
            # XXX - don't use recursion!@#$
            name = name + unpack_name(buf, ptr)[0] + '.'
            break
        else:
            off += 1
            name = name + buf[off:off+n] + '.'
            if len(name) > 255:
                raise dpkt.UnpackError('name longer than 255 bytes')
            off += n
    return name.strip('.'), off

class DNS(dpkt.Packet):
    __hdr__ = (
        ('id', 'H', 0),
        ('op', 'H', DNS_RD),	# recursive query
        # XXX - lists of query, RR objects
        ('qd', 'H', []),
        ('an', 'H', []),
        ('ns', 'H', []),
        ('ar', 'H', [])
        )
    def get_qr(self):
        return int((self.op & DNS_QR) == DNS_QR)
    def set_qr(self, v):
        if v: self.op |= DNS_QR
        else: self.op &= ~DNS_QR
    qr = property(get_qr, set_qr)

    def get_opcode(self):
        return (self.op >> 11) & 0xf
    def set_opcode(self, v):
        self.op = (self.op & ~0x7800) | ((v & 0xf) << 11)
    opcode = property(get_opcode, set_opcode)

    def get_aa(self):
        return int((self.op & DNS_AA) == DNS_AA)
    def set_aa(self, v):
        if v: self.op |= DNS_AA
        else: self.op &= ~DNS_AA
    aa = property(get_aa, set_aa)

    def get_rd(self):
        return int((self.op & DNS_RD) == DNS_RD)
    def set_rd(self,v):
        if v: self.op |= DNS_RD
        else: self.op &= ~DNS_RD
    rd = property(get_rd, set_rd)

    def get_ra(self):
        return int((self.op & DNS_RA) == DNS_RA)
    def set_ra(self,v):
        if v: self.op |= DNS_RA
        else: self.op &= ~DNS_RA
    ra = property(get_ra, set_ra)

    def get_zero(self):
        return int((self.op & DNS_Z) == DNS_Z)
    def set_zero(self, v):
        if v: self.op |= DNS_Z
        else: self.op &= ~DNS_Z
    zero = property(get_zero, set_zero)

    def get_rcode(self):
        return self.op & 0xf
    def set_rcode(self, v):
        self.op = (self.op & ~0xf) | (v & 0xf)
    rcode = property(get_rcode, set_rcode)

    class Q(dpkt.Packet):
        """DNS question."""
        __hdr__ = (
            ('name', '1025s', ''),
            ('type', 'H', DNS_A),
            ('cls', 'H', DNS_IN)
            )
        # XXX - suk
        def __len__(self):
            raise NotImplementedError
        __str__ = __len__
        def unpack(self, buf):
            raise NotImplementedError

    class RR(Q):
        """DNS resource record."""
        __hdr__ = (
            ('name', '1025s', ''),
            ('type', 'H', DNS_A),
            ('cls', 'H', DNS_IN),
            ('ttl', 'I', 0),
            ('rlen', 'H', 4),
            ('rdata', 's', '')
            )
        def pack_rdata(self, off, label_ptrs):
            # XXX - yeah, this sux
            if self.rdata:
                return self.rdata
            if self.type == DNS_A:
                return self.ip
            elif self.type == DNS_NS:
                return pack_name(self.nsname, off, label_ptrs)
            elif self.type == DNS_CNAME:
                return pack_name(self.cname, off, label_ptrs)
            elif self.type == DNS_PTR:
                return pack_name(self.ptrname, off, label_ptrs)
            elif self.type == DNS_SOA:
                l = []
                l.append(pack_name(self.mname, off, label_ptrs))
                l.append(pack_name(self.rname, off + len(l[0]), label_ptrs))
                l.append(struct.pack('>IIIII', self.serial, self.refresh,
                                     self.retry, self.expire, self.minimum))
                return ''.join(l)
            elif self.type == DNS_MX:
                return struct.pack('>H', self.preference) + \
                       pack_name(self.mxname, off + 2, label_ptrs)
            elif self.type == DNS_TXT or self.type == DNS_HINFO:
                return ''.join([ '%s%s' % (chr(len(x)), x)
                                 for x in self.text ])
            elif self.type == DNS_AAAA:
                return self.ip6
            elif self.type == DNS_SRV:
                return struct.pack('>HHH', self.priority, self.weight, self.port) + \
                       pack_name(self.srvname, off + 6, label_ptrs)
        
        def unpack_rdata(self, buf, off):
            if self.type == DNS_A:
                self.ip = self.rdata
            elif self.type == DNS_NS:
                self.nsname, off = unpack_name(buf, off)
            elif self.type == DNS_CNAME:
                self.cname, off = unpack_name(buf, off)
            elif self.type == DNS_PTR:
                self.ptrname, off = unpack_name(buf, off)
            elif self.type == DNS_SOA:
                self.mname, off = unpack_name(buf, off)
                self.rname, off = unpack_name(buf, off)
                self.serial, self.refresh, self.retry, self.expire, \
                    self.minimum = struct.unpack('>IIIII', buf[off:off+20])
            elif self.type == DNS_MX:
                self.preference = struct.unpack('>H', self.rdata[:2])
                self.mxname, off = unpack_name(buf, off+2)
            elif self.type == DNS_TXT or self.type == DNS_HINFO:
                self.text = []
                buf = self.rdata
                while buf:
                    n = ord(buf[0])
                    self.text.append(buf[1:1+n])
                    buf = buf[1+n:]
            elif self.type == DNS_AAAA:
                self.ip6 = self.rdata
            elif self.type == DNS_SRV:
                self.priority, self.weight, self.port = \
                    struct.unpack('>HHH', self.rdata[:6])
                self.srvname, off = unpack_name(buf, off+6)
    
    def pack_q(self, buf, q):
        """Append packed DNS question and return buf."""
        return buf + pack_name(q.name, len(buf), self.label_ptrs) + \
               struct.pack('>HH', q.type, q.cls)
    
    def unpack_q(self, buf, off):
        """Return DNS question and new offset."""
        q = self.Q()
        q.name, off = unpack_name(buf, off)
        q.type, q.cls = struct.unpack('>HH', buf[off:off+4])
        off += 4
        return q, off

    def pack_rr(self, buf, rr):
        """Append packed DNS RR and return buf."""
        name = pack_name(rr.name, len(buf), self.label_ptrs)
        rdata = rr.pack_rdata(len(buf) + len(name) + 10, self.label_ptrs)
        return buf + name + struct.pack('>HHIH', rr.type, rr.cls, rr.ttl,
                                        len(rdata)) + rdata
    
    def unpack_rr(self, buf, off):
        """Return DNS RR and new offset."""
        rr = self.RR()
        rr.name, off = unpack_name(buf, off)
        rr.type, rr.cls, rr.ttl, rdlen = struct.unpack('>HHIH', buf[off:off+10])
        off += 10
        rr.rdata = buf[off:off+rdlen]
        rr.unpack_rdata(buf, off)
        off += rdlen
        return rr, off
    
    def unpack(self, buf):
        dpkt.Packet.unpack(self, buf)
        off = self.__hdr_len__
        cnt = self.qd
        self.qd = []
        for i in range(cnt):
            q, off = self.unpack_q(buf, off)
            self.qd.append(q)
        for x in ('an', 'ns', 'ar'):
            cnt = getattr(self, x, 0)
            setattr(self, x, [])
            for i in range(cnt):
                rr, off = self.unpack_rr(buf, off)
                getattr(self, x).append(rr)
        self.data = ''

    def __len__(self):
        # XXX - cop out
        return len(str(self))

    def __str__(self):
        # XXX - compress names on the fly
        self.label_ptrs = {}
        buf = struct.pack(self.__hdr_fmt__, self.id, self.op, len(self.qd),
                          len(self.an), len(self.ns), len(self.ar))
        for q in self.qd:
            buf = self.pack_q(buf, q)
        for x in ('an', 'ns', 'ar'):
            for rr in getattr(self, x):
                buf = self.pack_rr(buf, rr)
        del self.label_ptrs
        return buf

if __name__ == '__main__':
    import unittest
    from ip import IP

    class DNSTestCase(unittest.TestCase):
        def test_basic(self):
            s = 'E\x00\x02\x08\xc15\x00\x00\x80\x11\x92aBk0\x01Bk0w\x005\xc07\x01\xf4\xda\xc2d\xd2\x81\x80\x00\x01\x00\x03\x00\x0b\x00\x0b\x03www\x06google\x03com\x00\x00\x01\x00\x01\xc0\x0c\x00\x05\x00\x01\x00\x00\x03V\x00\x17\x03www\x06google\x06akadns\x03net\x00\xc0,\x00\x01\x00\x01\x00\x00\x01\xa3\x00\x04@\xe9\xabh\xc0,\x00\x01\x00\x01\x00\x00\x01\xa3\x00\x04@\xe9\xabc\xc07\x00\x02\x00\x01\x00\x00KG\x00\x0c\x04usw5\x04akam\xc0>\xc07\x00\x02\x00\x01\x00\x00KG\x00\x07\x04usw6\xc0t\xc07\x00\x02\x00\x01\x00\x00KG\x00\x07\x04usw7\xc0t\xc07\x00\x02\x00\x01\x00\x00KG\x00\x08\x05asia3\xc0t\xc07\x00\x02\x00\x01\x00\x00KG\x00\x05\x02za\xc07\xc07\x00\x02\x00\x01\x00\x00KG\x00\x0f\x02zc\x06akadns\x03org\x00\xc07\x00\x02\x00\x01\x00\x00KG\x00\x05\x02zf\xc07\xc07\x00\x02\x00\x01\x00\x00KG\x00\x05\x02zh\xc0\xd5\xc07\x00\x02\x00\x01\x00\x00KG\x00\x07\x04eur3\xc0t\xc07\x00\x02\x00\x01\x00\x00KG\x00\x07\x04use2\xc0t\xc07\x00\x02\x00\x01\x00\x00KG\x00\x07\x04use4\xc0t\xc0\xc1\x00\x01\x00\x01\x00\x00\xfb4\x00\x04\xd0\xb9\x84\xb0\xc0\xd2\x00\x01\x00\x01\x00\x001\x0c\x00\x04?\xf1\xc76\xc0\xed\x00\x01\x00\x01\x00\x00\xfb4\x00\x04?\xd7\xc6S\xc0\xfe\x00\x01\x00\x01\x00\x001\x0c\x00\x04?\xd00.\xc1\x0f\x00\x01\x00\x01\x00\x00\n\xdf\x00\x04\xc1-\x01g\xc1"\x00\x01\x00\x01\x00\x00\x101\x00\x04?\xd1\xaa\x88\xc15\x00\x01\x00\x01\x00\x00\r\x1a\x00\x04PCC\xb6\xc0o\x00\x01\x00\x01\x00\x00\x10\x7f\x00\x04?\xf1I\xd6\xc0\x87\x00\x01\x00\x01\x00\x00\n\xdf\x00\x04\xce\x84dl\xc0\x9a\x00\x01\x00\x01\x00\x00\n\xdf\x00\x04A\xcb\xea\x1b\xc0\xad\x00\x01\x00\x01\x00\x00\x0b)\x00\x04\xc1l\x9a\t'
            ip = IP(s)
            dns = DNS(ip.udp.data)
            self.failUnless(dns.qd[0].name == 'www.google.com' and
                            dns.an[1].name == 'www.google.akadns.net')
            s = '\x05\xf5\x01\x00\x00\x01\x00\x00\x00\x00\x00\x00\x03www\x03cnn\x03com\x00\x00\x01\x00\x01'
            dns = DNS(s)
            self.failUnless(s == str(dns))

        def test_PTR(self):
            s = 'g\x02\x81\x80\x00\x01\x00\x01\x00\x03\x00\x00\x011\x011\x03211\x03141\x07in-addr\x04arpa\x00\x00\x0c\x00\x01\xc0\x0c\x00\x0c\x00\x01\x00\x00\r6\x00$\x07default\nv-umce-ifs\x05umnet\x05umich\x03edu\x00\xc0\x0e\x00\x02\x00\x01\x00\x00\r6\x00\r\x06shabby\x03ifs\xc0O\xc0\x0e\x00\x02\x00\x01\x00\x00\r6\x00\x0f\x0cfish-license\xc0m\xc0\x0e\x00\x02\x00\x01\x00\x00\r6\x00\x0b\x04dns2\x03itd\xc0O'
            dns = DNS(s)
            self.failUnless(dns.qd[0].name == '1.1.211.141.in-addr.arpa' and
                            dns.an[0].ptrname == 'default.v-umce-ifs.umnet.umich.edu' and
                            dns.ns[0].nsname == 'shabby.ifs.umich.edu' and
                            dns.ns[1].ttl == 3382L and
                            dns.ns[2].nsname == 'dns2.itd.umich.edu')
            self.failUnless(s == str(dns))
    
        def test_pack_name(self):
            # Empty name is \0
            x = pack_name('', 0, {})
            self.assertEqual(x, '\0')

    unittest.main()



# $Id: gre.py 75 2010-08-03 14:42:19Z jon.oberheide $

"""Generic Routing Encapsulation."""

import struct
import dpkt

GRE_CP = 0x8000  # Checksum Present
GRE_RP = 0x4000  # Routing Present
GRE_KP = 0x2000  # Key Present
GRE_SP = 0x1000  # Sequence Present
GRE_SS = 0x0800  # Strict Source Route
GRE_AP = 0x0080  # Acknowledgment Present

GRE_opt_fields = (
    (GRE_CP|GRE_RP, 'sum', 'H'), (GRE_CP|GRE_RP, 'off', 'H'),
    (GRE_KP, 'key', 'I'), (GRE_SP, 'seq', 'I'), (GRE_AP, 'ack', 'I')
    )
class GRE(dpkt.Packet):
    __hdr__ = (
        ('flags', 'H', 0),
        ('p', 'H', 0x0800), # ETH_TYPE_IP
        )
    _protosw = {}
    sre = ()
    def get_v(self):
        return self.flags & 0x7
    def set_v(self, v):
        self.flags = (self.flags & ~0x7) | (v & 0x7)
    v = property(get_v, set_v)

    def get_recur(self):
        return (self.flags >> 5) & 0x7
    def set_recur(self, v):
        self.flags = (self.flags & ~0xe0) | ((v & 0x7) << 5)
    recur = property(get_recur, set_recur)
    
    class SRE(dpkt.Packet):
        __hdr__ = [
            ('family', 'H', 0),
            ('off', 'B', 0),
            ('len', 'B', 0)
            ]
        def unpack(self, buf):
            dpkt.Packet.unpack(self, buf)
            self.data = self.data[:self.len]

    def opt_fields_fmts(self):
        if self.v == 0:
            fields, fmts = [], []
            opt_fields = GRE_opt_fields
        else:
            fields, fmts = [ 'len', 'callid' ], [ 'H', 'H' ]
            opt_fields = GRE_opt_fields[-2:]
        for flags, field, fmt in opt_fields:
            if self.flags & flags:
                fields.append(field)
                fmts.append(fmt)
        return fields, fmts
    
    def unpack(self, buf):
        dpkt.Packet.unpack(self, buf)
        fields, fmts = self.opt_fields_fmts()
        if fields:
            fmt = ''.join(fmts)
            fmtlen = struct.calcsize(fmt)
            vals = struct.unpack(fmt, self.data[:fmtlen])
            self.data = self.data[fmtlen:]
            self.__dict__.update(dict(zip(fields, vals)))
        if self.flags & GRE_RP:
            l = []
            while True:
                sre = self.SRE(self.data)
                self.data = self.data[len(sre):]
                l.append(sre)
                if not sre.len:
                    break
            self.sre = l
        self.data = ethernet.Ethernet._typesw[self.p](self.data)
        setattr(self, self.data.__class__.__name__.lower(), self.data)
    
    def __len__(self):
        opt_fmtlen = struct.calcsize(''.join(self.opt_fields_fmts()[1]))
        return self.__hdr_len__ + opt_fmtlen + \
               sum(map(len, self.sre)) + len(self.data)

    # XXX - need to fix up repr to display optional fields...
    
    def __str__(self):
        fields, fmts = self.opt_fields_fmts()
        if fields:
            vals = []
            for f in fields:
                vals.append(getattr(self, f))
            opt_s = struct.pack(''.join(fmts), *vals)
        else:
            opt_s = ''
        return self.pack_hdr() + opt_s + ''.join(map(str, self.sre)) + \
               str(self.data)

# XXX - auto-load GRE dispatch table from Ethernet dispatch table
import ethernet
GRE._protosw.update(ethernet.Ethernet._typesw)



# $Id: aim.py 23 2006-11-08 15:45:33Z dugsong $

"""AOL Instant Messenger."""

import dpkt
import struct

# OSCAR: http://iserverd1.khstu.ru/oscar/

class FLAP(dpkt.Packet):
    __hdr__ = (
        ('ast', 'B', 0x2a),	# '*'
        ('type', 'B', 0),
        ('seq', 'H', 0),
        ('len', 'H', 0)
    )
    def unpack(self, buf):
        dpkt.Packet.unpack(self, buf)
        if self.ast != 0x2a:
            raise dpkt.UnpackError('invalid FLAP header')
        if len(self.data) < self.len:
            raise dpkt.NeedData, '%d left, %d needed' % (len(self.data), self.len)

class SNAC(dpkt.Packet):
    __hdr__ = (
        ('family', 'H', 0),
        ('subtype', 'H', 0),
        ('flags', 'H', 0),
        ('reqid', 'I', 0)
        )

def tlv(buf):
    n = 4
    try:
        t, l = struct.unpack('>HH', buf[:n])
    except struct.error:
        raise dpkt.UnpackError
    v = buf[n:n+l]
    if len(v) < l:
        raise dpkt.NeedData
    buf = buf[n+l:]
    return (t,l,v, buf)

# TOC 1.0: http://jamwt.com/Py-TOC/PROTOCOL

# TOC 2.0: http://www.firestuff.org/projects/firetalk/doc/toc2.txt




# $Id: ntp.py 48 2008-05-27 17:31:15Z yardley $

"""Network Time Protocol."""

import dpkt

# NTP v4

# Leap Indicator (LI) Codes
NO_WARNING		= 0
LAST_MINUTE_61_SECONDS	= 1
LAST_MINUTE_59_SECONDS	= 2
ALARM_CONDITION		= 3

# Mode Codes
RESERVED		= 0
SYMMETRIC_ACTIVE	= 1
SYMMETRIC_PASSIVE	= 2
CLIENT			= 3
SERVER			= 4
BROADCAST		= 5
CONTROL_MESSAGE		= 6
PRIVATE			= 7

class NTP(dpkt.Packet):
    __hdr__ = (
        ('flags', 'B', 0),
        ('stratum', 'B', 0),
        ('interval', 'B', 0),
        ('precision', 'B', 0),
        ('delay', 'I', 0),
        ('dispersion', 'I', 0),
        ('id', '4s', 0),
        ('update_time', '8s', 0),
        ('originate_time', '8s', 0),
        ('receive_time', '8s', 0),
        ('transmit_time', '8s', 0)
        )

    def _get_v(self):
        return (self.flags >> 3) & 0x7
    def _set_v(self, v):
        self.flags = (self.flags & ~0x38) | ((v & 0x7) << 3)
    v = property(_get_v, _set_v)

    def _get_li(self):
        return (self.flags >> 6) & 0x3
    def _set_li(self, li):
        self.flags = (self.flags & ~0xc0) | ((li & 0x3) << 6)
    li = property(_get_li, _set_li)
    
    def _get_mode(self):
        return (self.flags & 0x7)
    def _set_mode(self, mode):
        self.flags = (self.flags & ~0x7) | (mode & 0x7)
    mode = property(_get_mode, _set_mode)

if __name__ == '__main__':
    import unittest

    class NTPTestCase(unittest.TestCase):
        def testPack(self):
            n = NTP(self.s)
            self.failUnless(self.s == str(n))

        def testUnpack(self):
            n = NTP(self.s)
            self.failUnless(n.li == NO_WARNING)
            self.failUnless(n.v == 4)
            self.failUnless(n.mode == SERVER)
            self.failUnless(n.stratum == 2)
            self.failUnless(n.id == '\xc1\x02\x04\x02')

            # test get/set functions
            n.li = ALARM_CONDITION
            n.v = 3
            n.mode = CLIENT
            self.failUnless(n.li == ALARM_CONDITION)
            self.failUnless(n.v == 3)
            self.failUnless(n.mode == CLIENT)

        s = '\x24\x02\x04\xef\x00\x00\x00\x84\x00\x00\x33\x27\xc1\x02\x04\x02\xc8\x90\xec\x11\x22\xae\x07\xe5\xc8\x90\xf9\xd9\xc0\x7e\x8c\xcd\xc8\x90\xf9\xd9\xda\xc5\xb0\x78\xc8\x90\xf9\xd9\xda\xc6\x8a\x93'
    unittest.main()



# $Id: ospf.py 23 2006-11-08 15:45:33Z dugsong $

"""Open Shortest Path First."""

import dpkt

AUTH_NONE = 0
AUTH_PASSWORD = 1
AUTH_CRYPTO = 2

class OSPF(dpkt.Packet):
    __hdr__ = (
        ('v', 'B', 0),
        ('type', 'B', 0),
        ('len', 'H', 0),
        ('router', 'I', 0),
        ('area', 'I', 0),
        ('sum', 'H', 0),
        ('atype', 'H', 0),
        ('auth', '8s', '')
        )
    def __str__(self):
        if not self.sum:
            self.sum = dpkt.in_cksum(dpkt.Packet.__str__(self))
        return dpkt.Packet.__str__(self)



# $Id: rtp.py 23 2006-11-08 15:45:33Z dugsong $

"""Real-Time Transport Protocol"""

from dpkt import Packet

# version 1100 0000 0000 0000 ! 0xC000  14
# p       0010 0000 0000 0000 ! 0x2000  13
# x       0001 0000 0000 0000 ! 0x1000  12
# cc      0000 1111 0000 0000 ! 0x0F00   8
# m       0000 0000 1000 0000 ! 0x0080   7
# pt      0000 0000 0111 1111 ! 0x007F   0
#

_VERSION_MASK= 0xC000
_P_MASK     = 0x2000
_X_MASK     = 0x1000
_CC_MASK    = 0x0F00
_M_MASK     = 0x0080
_PT_MASK    = 0x007F
_VERSION_SHIFT=14
_P_SHIFT    = 13
_X_SHIFT    = 12
_CC_SHIFT   = 8
_M_SHIFT    = 7
_PT_SHIFT   = 0

VERSION = 2

class RTP(Packet):
    __hdr__ = (
        ('_type', 'H',      0x8000),
        ('seq',     'H',    0),
        ('ts',      'I',    0),
        ('ssrc',    'I',    0),
    )
    csrc = ''

    def _get_version(self): return (self._type&_VERSION_MASK)>>_VERSION_SHIFT
    def _set_version(self, ver):
        self._type = (ver << _VERSION_SHIFT) | (self._type & ~_VERSION_MASK)
    def _get_p(self): return (self._type & _P_MASK) >> _P_SHIFT
    def _set_p(self, p): self._type = (p << _P_SHIFT) | (self._type & ~_P_MASK)
    def _get_x(self): return (self._type & _X_MASK) >> _X_SHIFT
    def _set_x(self, x): self._type = (x << _X_SHIFT) | (self._type & ~_X_MASK)
    def _get_cc(self): return (self._type & _CC_MASK) >> _CC_SHIFT
    def _set_cc(self, cc): self._type = (cc<<_CC_SHIFT)|(self._type&~_CC_MASK)
    def _get_m(self): return (self._type & _M_MASK) >> _M_SHIFT
    def _set_m(self, m): self._type = (m << _M_SHIFT) | (self._type & ~_M_MASK)
    def _get_pt(self): return (self._type & _PT_MASK) >> _PT_SHIFT
    def _set_pt(self, m): self._type = (m << _PT_SHIFT)|(self._type&~_PT_MASK)

    version = property(_get_version, _set_version)
    p = property(_get_p, _set_p)
    x = property(_get_x, _set_x)
    cc = property(_get_cc, _set_cc)
    m = property(_get_m, _set_m)
    pt = property(_get_pt, _set_pt)

    def __len__(self):
        return self.__hdr_len__ + len(self.csrc) + len(self.data)

    def __str__(self):
        return self.pack_hdr() + self.csrc + str(self.data)

    def unpack(self, buf):
        super(RTP, self).unpack(buf)
        self.csrc = buf[self.__hdr_len__:self.__hdr_len__ + self.cc * 4]
        self.data = buf[self.__hdr_len__ + self.cc * 4:]




# $Id: dhcp.py 23 2006-11-08 15:45:33Z dugsong $

"""Dynamic Host Configuration Protocol."""

import arp, dpkt

DHCP_OP_REQUEST = 1
DHCP_OP_REPLY = 2

DHCP_MAGIC = 0x63825363

# DHCP option codes
DHCP_OPT_NETMASK =         1 # I: subnet mask
DHCP_OPT_TIMEOFFSET =      2
DHCP_OPT_ROUTER =          3 # s: list of router ips
DHCP_OPT_TIMESERVER =      4
DHCP_OPT_NAMESERVER =      5
DHCP_OPT_DNS_SVRS =        6 # s: list of DNS servers
DHCP_OPT_LOGSERV =         7
DHCP_OPT_COOKIESERV =      8
DHCP_OPT_LPRSERV =         9
DHCP_OPT_IMPSERV =         10
DHCP_OPT_RESSERV =         11
DHCP_OPT_HOSTNAME =        12 # s: client hostname
DHCP_OPT_BOOTFILESIZE =    13
DHCP_OPT_DUMPFILE =        14
DHCP_OPT_DOMAIN =          15 # s: domain name
DHCP_OPT_SWAPSERV =        16
DHCP_OPT_ROOTPATH =        17
DHCP_OPT_EXTENPATH =       18
DHCP_OPT_IPFORWARD =       19
DHCP_OPT_SRCROUTE =        20
DHCP_OPT_POLICYFILTER =    21
DHCP_OPT_MAXASMSIZE =      22
DHCP_OPT_IPTTL =           23
DHCP_OPT_MTUTIMEOUT =      24
DHCP_OPT_MTUTABLE =        25
DHCP_OPT_MTUSIZE =         26
DHCP_OPT_LOCALSUBNETS =    27
DHCP_OPT_BROADCASTADDR =   28
DHCP_OPT_DOMASKDISCOV =    29
DHCP_OPT_MASKSUPPLY =      30
DHCP_OPT_DOROUTEDISC =     31
DHCP_OPT_ROUTERSOLICIT =   32
DHCP_OPT_STATICROUTE =     33
DHCP_OPT_TRAILERENCAP =    34
DHCP_OPT_ARPTIMEOUT =      35
DHCP_OPT_ETHERENCAP =      36
DHCP_OPT_TCPTTL =          37
DHCP_OPT_TCPKEEPALIVE =    38
DHCP_OPT_TCPALIVEGARBAGE = 39
DHCP_OPT_NISDOMAIN =       40
DHCP_OPT_NISSERVERS =      41
DHCP_OPT_NISTIMESERV =     42
DHCP_OPT_VENDSPECIFIC =    43
DHCP_OPT_NBNS =            44
DHCP_OPT_NBDD =            45
DHCP_OPT_NBTCPIP =         46
DHCP_OPT_NBTCPSCOPE =      47
DHCP_OPT_XFONT =           48
DHCP_OPT_XDISPLAYMGR =     49
DHCP_OPT_REQ_IP =          50 # I: IP address
DHCP_OPT_LEASE_SEC =       51 # I: lease seconds
DHCP_OPT_OPTIONOVERLOAD =  52
DHCP_OPT_MSGTYPE =         53 # B: message type
DHCP_OPT_SERVER_ID =       54 # I: server IP address
DHCP_OPT_PARAM_REQ =       55 # s: list of option codes
DHCP_OPT_MESSAGE =         56
DHCP_OPT_MAXMSGSIZE =      57
DHCP_OPT_RENEWTIME =       58
DHCP_OPT_REBINDTIME =      59
DHCP_OPT_VENDOR_ID =       60 # s: vendor class id
DHCP_OPT_CLIENT_ID =       61 # Bs: idtype, id (idtype 0: FQDN, idtype 1: MAC)
DHCP_OPT_NISPLUSDOMAIN =   64
DHCP_OPT_NISPLUSSERVERS =  65
DHCP_OPT_MOBILEIPAGENT =   68
DHCP_OPT_SMTPSERVER =      69
DHCP_OPT_POP3SERVER =      70
DHCP_OPT_NNTPSERVER =      71
DHCP_OPT_WWWSERVER =       72
DHCP_OPT_FINGERSERVER =    73
DHCP_OPT_IRCSERVER =       74
DHCP_OPT_STSERVER =        75
DHCP_OPT_STDASERVER =      76

# DHCP message type values
DHCPDISCOVER = 1
DHCPOFFER = 2
DHCPREQUEST = 3
DHCPDECLINE = 4
DHCPACK = 5
DHCPNAK = 6
DHCPRELEASE = 7
DHCPINFORM = 8

class DHCP(dpkt.Packet):
    __hdr__ = (
        ('op', 'B', DHCP_OP_REQUEST),
        ('hrd', 'B', arp.ARP_HRD_ETH),  # just like ARP.hrd
        ('hln', 'B', 6),		# and ARP.hln
        ('hops', 'B', 0),
        ('xid', 'I', 0xdeadbeefL),
        ('secs', 'H', 0),
        ('flags', 'H', 0),
        ('ciaddr', 'I', 0),
        ('yiaddr', 'I', 0),
        ('siaddr', 'I', 0),
        ('giaddr', 'I', 0),
        ('chaddr', '16s', 16 * '\x00'),
        ('sname', '64s', 64 * '\x00'),
        ('file', '128s', 128 * '\x00'),
        ('magic', 'I', DHCP_MAGIC),
        )
    opts = (
        (DHCP_OPT_MSGTYPE, chr(DHCPDISCOVER)),
        (DHCP_OPT_PARAM_REQ, ''.join(map(chr, (DHCP_OPT_REQ_IP,
                                               DHCP_OPT_ROUTER,
                                               DHCP_OPT_NETMASK,
                                               DHCP_OPT_DNS_SVRS))))
        )				# list of (type, data) tuples

    def __len__(self):
        return self.__hdr_len__ + \
               sum([ 2 + len(o[1]) for o in self.opts ]) + 1 + len(self.data)
    
    def __str__(self):
        return self.pack_hdr() + self.pack_opts() + str(self.data)
    
    def pack_opts(self):
        """Return packed options string."""
        if not self.opts:
            return ''
        l = []
        for t, data in self.opts:
            l.append('%s%s%s' % (chr(t), chr(len(data)), data))
        l.append('\xff')
        return ''.join(l)
    
    def unpack(self, buf):
        dpkt.Packet.unpack(self, buf)
        self.chaddr = self.chaddr[:self.hln]
        buf = self.data
        l = []
        while buf:
            t = ord(buf[0])
            if t == 0xff:
                buf = buf[1:]
                break
            elif t == 0:
                buf = buf[1:]
            else:
                n = ord(buf[1])
                l.append((t, buf[2:2+n]))
                buf = buf[2+n:]
        self.opts = l
        self.data = buf

if __name__ == '__main__':
    import unittest

    class DHCPTestCast(unittest.TestCase):
        def test_DHCP(self):
            s = '\x01\x01\x06\x00\xadS\xc8c\xb8\x87\x80\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x02U\x82\xf3\xa6\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00c\x82Sc5\x01\x01\xfb\x01\x01=\x07\x01\x00\x02U\x82\xf3\xa62\x04\n\x00\x01e\x0c\tGuinevere<\x08MSFT 5.07\n\x01\x0f\x03\x06,./\x1f!+\xff\x00\x00\x00\x00\x00'
            dhcp = DHCP(s)
            self.failUnless(s == str(dhcp))

    unittest.main()




# $Id: sccp.py 23 2006-11-08 15:45:33Z dugsong $

"""Cisco Skinny Client Control Protocol."""

import dpkt

KEYPAD_BUTTON		= 0x00000003
OFF_HOOK		= 0x00000006
ON_HOOK			= 0x00000007
OPEN_RECEIVE_CHANNEL_ACK= 0x00000022
START_TONE		= 0x00000082
STOP_TONE		= 0x00000083
SET_LAMP		= 0x00000086
SET_SPEAKER_MODE	= 0x00000088
START_MEDIA_TRANSMIT	= 0x0000008A
STOP_MEDIA_TRANSMIT	= 0x0000008B
CALL_INFO		= 0x0000008F
DEFINE_TIME_DATE	= 0x00000094
DISPLAY_TEXT		= 0x00000099
OPEN_RECEIVE_CHANNEL	= 0x00000105
CLOSE_RECEIVE_CHANNEL	= 0x00000106
SELECT_SOFTKEYS		= 0x00000110
CALL_STATE		= 0x00000111
DISPLAY_PROMPT_STATUS	= 0x00000112
CLEAR_PROMPT_STATUS	= 0x00000113
ACTIVATE_CALL_PLANE	= 0x00000116

class ActivateCallPlane(dpkt.Packet):
    __byte_order__ = '<'
    __hdr__ = (
        ('line_instance', 'I', 0),
        )

class CallInfo(dpkt.Packet):
    __byte_order__ = '<'
    __hdr__ = (
        ('calling_party_name', '40s', ''),
        ('calling_party', '24s', ''),
        ('called_party_name', '40s', ''),
        ('called_party', '24s', ''),
        ('line_instance', 'I', 0),
        ('call_id', 'I', 0),
        ('call_type', 'I', 0),
        ('orig_called_party_name', '40s', ''),
        ('orig_called_party', '24s', '')
        )

class CallState(dpkt.Packet):
    __byte_order__ = '<'
    __hdr__ = (
        ('call_state', 'I', 12), # 12: Proceed, 15: Connected
        ('line_instance', 'I', 1),
        ('call_id', 'I', 0)
        )

class ClearPromptStatus(dpkt.Packet):
    __byte_order__ = '<'
    __hdr__ = (
        ('line_instance', 'I', 1),
        ('call_id', 'I', 0)
        )

class CloseReceiveChannel(dpkt.Packet):
    __byte_order__ = '<'
    __hdr__ = (
        ('conference_id', 'I', 0),
        ('passthruparty_id', 'I', 0),
        )
    
class DisplayPromptStatus(dpkt.Packet):
    __byte_order__ = '<'
    __hdr__ = (
        ('msg_timeout', 'I', 0),
        ('display_msg', '32s', ''),
        ('line_instance', 'I', 1),
        ('call_id', 'I', 0)
        )
    
class DisplayText(dpkt.Packet):
    __byte_order__ = '<'
    __hdr__ = (
        ('display_msg', '36s', ''),
        )

class KeypadButton(dpkt.Packet):
    __byte_order__ = '<'
    __hdr__ = (
        ('button', 'I', 0),
        )
    
class OpenReceiveChannel(dpkt.Packet):
    __byte_order__ = '<'
    __hdr__ = (
        ('conference_id', 'I', 0),
        ('passthruparty_id', 'I', 0),
        ('ms_packet', 'I', 0),
        ('payload_capability', 'I', 4), # 4: G.711 u-law 64k
        ('echo_cancel_type', 'I', 4),
        ('g723_bitrate', 'I', 0),
        )

class OpenReceiveChannelAck(dpkt.Packet):
    __byte_order__ = '<'
    __hdr__ = (
        ('channel_status', 'I', 0),
        ('ip', '4s', ''),
        ('port', 'I', 0),
        ('passthruparty_id', 'I', 0),
        )

class SelectStartKeys(dpkt.Packet):
    __byte_order__ = '<'
    __hdr__ = (
        ('line_id', 'I', 1),
        ('call_id', 'I', 0),
        ('softkey_set', 'I', 8),
        ('softkey_map', 'I', 0xffffffffL)
        )

class SetLamp(dpkt.Packet):
    __byte_order__ = '<'
    __hdr__ = (
        ('stimulus', 'I', 9), # 9: Line
        ('stimulus_instance', 'I', 1),
        ('lamp_mode', 'I', 1),
        )

class SetSpeakerMode(dpkt.Packet):
    __byte_order__ = '<'
    __hdr__ = (
        ('speaker', 'I', 2), # 2: SpeakerOff
        )

class StartMediaTransmission(dpkt.Packet):
    __byte_order__ = '<'
    __hdr__ = (
        ('conference_id', 'I', 0),
        ('passthruparty_id', 'I', 0),
        ('remote_ip', '4s', ''),
        ('remote_port', 'I', 0),
        ('ms_packet', 'I', 0),
        ('payload_capability', 'I', 4), # 4: G.711 u-law 64k
        ('precedence', 'I', 0),
        ('silence_suppression', 'I', 0),
        ('max_frames_per_pkt', 'I', 1),
        ('g723_bitrate', 'I', 0),
        )

class StartTone(dpkt.Packet):
    __byte_order__ = '<'
    __hdr__ = (
        ('tone', 'I', 0x24), # 0x24: AlertingTone
        )

class StopMediaTransmission(dpkt.Packet):
    __byte_order__ = '<'
    __hdr__ = (
        ('conference_id', 'I', 0),
        ('passthruparty_id', 'I', 0),
        )
    
class SCCP(dpkt.Packet):
    __byte_order__ = '<'
    __hdr__ = (
        ('len', 'I', 0),
        ('rsvd', 'I', 0),
        ('msgid', 'I', 0),
        ('msg', '0s', ''),
        )
    _msgsw = {
        KEYPAD_BUTTON:KeypadButton,
        OPEN_RECEIVE_CHANNEL_ACK:OpenReceiveChannelAck,
        START_TONE:StartTone,
        SET_LAMP:SetLamp,
        START_MEDIA_TRANSMIT:StartMediaTransmission,
        STOP_MEDIA_TRANSMIT:StopMediaTransmission,
        CALL_INFO:CallInfo,
        DISPLAY_TEXT:DisplayText,
        OPEN_RECEIVE_CHANNEL:OpenReceiveChannel,
        CLOSE_RECEIVE_CHANNEL:CloseReceiveChannel,
        CALL_STATE:CallState,
        DISPLAY_PROMPT_STATUS:DisplayPromptStatus,
        CLEAR_PROMPT_STATUS:ClearPromptStatus,
        ACTIVATE_CALL_PLANE:ActivateCallPlane,
        }
    def unpack(self, buf):
        dpkt.Packet.unpack(self, buf)
        n = self.len - 4
        if n > len(self.data):
            raise dpkt.NeedData('not enough data')
        self.msg, self.data = self.data[:n], self.data[n:]
        try:
            p = self._msgsw[self.msgid](self.msg)
            setattr(self, p.__class__.__name__.lower(), p)
        except (KeyError, dpkt.UnpackError):
            pass



# $Id: icmp.py 45 2007-08-03 00:05:22Z jon.oberheide $

"""Internet Control Message Protocol."""

import dpkt, ip

# Types (icmp_type) and codes (icmp_code) -
# http://www.iana.org/assignments/icmp-parameters

ICMP_CODE_NONE			= 0	# for types without codes
ICMP_ECHOREPLY		= 0	# echo reply
ICMP_UNREACH		= 3	# dest unreachable, codes:
ICMP_UNREACH_NET		= 0	# bad net
ICMP_UNREACH_HOST		= 1	# bad host
ICMP_UNREACH_PROTO		= 2	# bad protocol
ICMP_UNREACH_PORT		= 3	# bad port
ICMP_UNREACH_NEEDFRAG		= 4	# IP_DF caused drop
ICMP_UNREACH_SRCFAIL		= 5	# src route failed
ICMP_UNREACH_NET_UNKNOWN	= 6	# unknown net
ICMP_UNREACH_HOST_UNKNOWN	= 7	# unknown host
ICMP_UNREACH_ISOLATED		= 8	# src host isolated
ICMP_UNREACH_NET_PROHIB		= 9	# for crypto devs
ICMP_UNREACH_HOST_PROHIB	= 10	# ditto
ICMP_UNREACH_TOSNET		= 11	# bad tos for net
ICMP_UNREACH_TOSHOST		= 12	# bad tos for host
ICMP_UNREACH_FILTER_PROHIB	= 13	# prohibited access
ICMP_UNREACH_HOST_PRECEDENCE	= 14	# precedence error
ICMP_UNREACH_PRECEDENCE_CUTOFF	= 15	# precedence cutoff
ICMP_SRCQUENCH		= 4	# packet lost, slow down
ICMP_REDIRECT		= 5	# shorter route, codes:
ICMP_REDIRECT_NET		= 0	# for network
ICMP_REDIRECT_HOST		= 1	# for host
ICMP_REDIRECT_TOSNET		= 2	# for tos and net
ICMP_REDIRECT_TOSHOST		= 3	# for tos and host
ICMP_ALTHOSTADDR	= 6	# alternate host address
ICMP_ECHO		= 8	# echo service
ICMP_RTRADVERT		= 9	# router advertise, codes:
ICMP_RTRADVERT_NORMAL		= 0	# normal
ICMP_RTRADVERT_NOROUTE_COMMON	= 16	# selective routing
ICMP_RTRSOLICIT		= 10	# router solicitation
ICMP_TIMEXCEED		= 11	# time exceeded, code:
ICMP_TIMEXCEED_INTRANS		= 0	# ttl==0 in transit
ICMP_TIMEXCEED_REASS		= 1	# ttl==0 in reass
ICMP_PARAMPROB		= 12	# ip header bad
ICMP_PARAMPROB_ERRATPTR		= 0	# req. opt. absent
ICMP_PARAMPROB_OPTABSENT	= 1	# req. opt. absent
ICMP_PARAMPROB_LENGTH		= 2	# bad length
ICMP_TSTAMP		= 13	# timestamp request
ICMP_TSTAMPREPLY	= 14	# timestamp reply
ICMP_INFO		= 15	# information request
ICMP_INFOREPLY		= 16	# information reply
ICMP_MASK		= 17	# address mask request
ICMP_MASKREPLY		= 18	# address mask reply
ICMP_TRACEROUTE		= 30	# traceroute
ICMP_DATACONVERR	= 31	# data conversion error
ICMP_MOBILE_REDIRECT	= 32	# mobile host redirect
ICMP_IP6_WHEREAREYOU	= 33	# IPv6 where-are-you
ICMP_IP6_IAMHERE	= 34	# IPv6 i-am-here
ICMP_MOBILE_REG		= 35	# mobile registration req
ICMP_MOBILE_REGREPLY	= 36	# mobile registration reply
ICMP_DNS		= 37	# domain name request
ICMP_DNSREPLY		= 38	# domain name reply
ICMP_SKIP		= 39	# SKIP
ICMP_PHOTURIS		= 40	# Photuris
ICMP_PHOTURIS_UNKNOWN_INDEX	= 0	# unknown sec index
ICMP_PHOTURIS_AUTH_FAILED	= 1	# auth failed
ICMP_PHOTURIS_DECOMPRESS_FAILED	= 2	# decompress failed
ICMP_PHOTURIS_DECRYPT_FAILED	= 3	# decrypt failed
ICMP_PHOTURIS_NEED_AUTHN	= 4	# no authentication
ICMP_PHOTURIS_NEED_AUTHZ	= 5	# no authorization
ICMP_TYPE_MAX		= 40

class ICMP(dpkt.Packet):
    __hdr__ = (
        ('type', 'B', 8),
        ('code', 'B', 0),
        ('sum', 'H', 0)
        )
    class Echo(dpkt.Packet):
        __hdr__ = (('id', 'H', 0), ('seq', 'H', 0))
    class Quote(dpkt.Packet):
        __hdr__ = (('pad', 'I', 0),)
        def unpack(self, buf):
            dpkt.Packet.unpack(self, buf)
            self.data = self.ip = ip.IP(self.data)
    class Unreach(Quote):
        __hdr__ = (('pad', 'H', 0), ('mtu', 'H', 0))
    class Quench(Quote):
        pass
    class Redirect(Quote):
        __hdr__ = (('gw', 'I', 0),)
    class ParamProbe(Quote):
        __hdr__ = (('ptr', 'B', 0), ('pad1', 'B', 0), ('pad2', 'H', 0))
    class TimeExceed(Quote):
        pass
    
    _typesw = { 0:Echo, 3:Unreach, 4:Quench, 5:Redirect, 8:Echo,
                11:TimeExceed }
    
    def unpack(self, buf):
        dpkt.Packet.unpack(self, buf)
        try:
            self.data = self._typesw[self.type](self.data)
            setattr(self, self.data.__class__.__name__.lower(), self.data)
        except (KeyError, dpkt.UnpackError):
            pass

    def __str__(self):
        if not self.sum:
            self.sum = dpkt.in_cksum(dpkt.Packet.__str__(self))
        return dpkt.Packet.__str__(self)

if __name__ == '__main__':
    import unittest

    class ICMPTestCase(unittest.TestCase):
        def test_ICMP(self):
            s = '\x03\x0a\x6b\x19\x00\x00\x00\x00\x45\x00\x00\x28\x94\x1f\x00\x00\xe3\x06\x99\xb4\x23\x2b\x24\x00\xde\x8e\x84\x42\xab\xd1\x00\x50\x00\x35\xe1\x29\x20\xd9\x00\x00\x00\x22\x9b\xf0\xe2\x04\x65\x6b'
            icmp = ICMP(s)
            self.failUnless(str(icmp) == s)

    unittest.main()



# $Id: tns.py 23 2006-11-08 15:45:33Z dugsong $

"""Transparent Network Substrate."""

import dpkt

class TNS(dpkt.Packet):
    __hdr__ = (
    ('length', 'H', 0),
    ('pktsum', 'H', 0),
    ('type', 'B', 0),
    ('rsvd', 'B', 0),
    ('hdrsum', 'H', 0),
    ('msg', '0s', ''),
    )
    def unpack(self, buf):
        dpkt.Packet.unpack(self, buf)
        n = self.length - self.__hdr_len__
        if n > len(self.data):
            raise dpkt.NeedData('short message (missing %d bytes)' %
                                (n - len(self.data)))
        self.msg = self.data[:n]
        self.data = self.data[n:]




# $Id: h225.py 23 2006-11-08 15:45:33Z dugsong $

"""ITU-T H.225.0 Call Signaling."""

import dpkt, tpkt
import struct

# H225 Call Signaling
# 
# Call messages and information elements (IEs) are defined by Q.931:
# http://cvsup.de.openbsd.org/historic/comp/doc/standards/itu/Q/Q.931.ps.gz
#
# The User-to-User IEs of H225 are encoded by PER of ASN.1.

# Call Establishment Messages
ALERTING				= 1
CALL_PROCEEDING				= 2
CONNECT					= 7
CONNECT_ACKNOWLEDGE			= 15
PROGRESS				= 3
SETUP					= 5
SETUP_ACKNOWLEDGE			= 13

# Call Information Phase Messages
RESUME					= 38
RESUME_ACKNOWLEDGE			= 46
RESUME_REJECT				= 34
SUSPEND					= 37
SUSPEND_ACKNOWLEDGE			= 45
SUSPEND_REJECT				= 33
USER_INFORMATION			= 32

# Call Clearing Messages
DISCONNECT				= 69
RELEASE					= 77
RELEASE_COMPLETE			= 90
RESTART					= 70
RESTART_ACKNOWLEDGE			= 78

# Miscellaneous Messages
SEGMENT					= 96
CONGESTION_CONTROL			= 121
INFORMATION				= 123
NOTIFY					= 110
STATUS					= 125
STATUS_ENQUIRY				= 117

# Type 1 Single Octet Information Element IDs
RESERVED				= 128
SHIFT					= 144
CONGESTION_LEVEL			= 176
REPEAT_INDICATOR			= 208

# Type 2 Single Octet Information Element IDs
MORE_DATA				= 160
SENDING_COMPLETE			= 161

# Variable Length Information Element IDs 
SEGMENTED_MESSAGE			= 0
BEARER_CAPABILITY			= 4
CAUSE					= 8
CALL_IDENTITY				= 16
CALL_STATE				= 20
CHANNEL_IDENTIFICATION			= 24
PROGRESS_INDICATOR			= 30
NETWORK_SPECIFIC_FACILITIES		= 32
NOTIFICATION_INDICATOR			= 39
DISPLAY					= 40
DATE_TIME				= 41
KEYPAD_FACILITY				= 44
SIGNAL					= 52
INFORMATION_RATE			= 64
END_TO_END_TRANSIT_DELAY		= 66
TRANSIT_DELAY_SELECTION_AND_INDICATION	= 67
PACKET_LAYER_BINARY_PARAMETERS		= 68
PACKET_LAYER_WINDOW_SIZE		= 69
PACKET_SIZE				= 70
CLOSED_USER_GROUP			= 71
REVERSE_CHARGE_INDICATION		= 74
CALLING_PARTY_NUMBER			= 108
CALLING_PARTY_SUBADDRESS		= 109
CALLED_PARTY_NUMBER			= 112
CALLED_PARTY_SUBADDRESS			= 113
REDIRECTING_NUMBER			= 116
TRANSIT_NETWORK_SELECTION		= 120
RESTART_INDICATOR			= 121
LOW_LAYER_COMPATIBILITY			= 124
HIGH_LAYER_COMPATIBILITY		= 125
USER_TO_USER				= 126
ESCAPE_FOR_EXTENSION			= 127

class H225(dpkt.Packet):
    __hdr__ = (
        ('proto', 'B', 8),
        ('ref_len', 'B', 2)
        )

    def unpack(self, buf):
        # TPKT header
        self.tpkt = tpkt.TPKT(buf)
        if self.tpkt.v != 3: 
            raise dpkt.UnpackError('invalid TPKT version')
        if self.tpkt.rsvd != 0:
            raise dpkt.UnpackError('invalid TPKT reserved value')
        n = self.tpkt.len - self.tpkt.__hdr_len__
        if n > len(self.tpkt.data):
            raise dpkt.UnpackError('invalid TPKT length')
        buf = self.tpkt.data

        # Q.931 payload
        dpkt.Packet.unpack(self, buf)
        buf = buf[self.__hdr_len__:]
        self.ref_val = buf[:self.ref_len]
        buf = buf[self.ref_len:]
        self.type = struct.unpack('B', buf[:1])[0]
        buf = buf[1:]

        # Information Elements
        l = []
        while buf:
            ie = self.IE(buf)
            l.append(ie)
            buf = buf[len(ie):]
        self.data = l

    def __len__(self):
        return self.tpkt.__hdr_len__ + \
               self.__hdr_len__ + \
               sum(map(len, self.data))

    def __str__(self):
        return self.tpkt.pack_hdr() + \
               self.pack_hdr() + \
               self.ref_val + \
               struct.pack('B', self.type) + \
               ''.join(map(str, self.data))

    class IE(dpkt.Packet):
        __hdr__ = (
            ('type', 'B', 0),
            )

        def unpack(self, buf):
            dpkt.Packet.unpack(self, buf)
            buf = buf[self.__hdr_len__:]

            # single-byte IE
            if self.type & 0x80:
                self.len = 0
                self.data = None
            # multi-byte IE
            else:
                # special PER-encoded UUIE
                if self.type == USER_TO_USER:
                    self.len = struct.unpack('>H', buf[:2])[0]
                    buf = buf[2:]
                # normal TLV-like IE
                else:
                    self.len = struct.unpack('B', buf[:1])[0]
                    buf = buf[1:]
                self.data = buf[:self.len]

        def __len__(self):
            if self.type & 0x80:
                n = 0
            else:
                if self.type == USER_TO_USER:
                    n = 2
                else:
                    n = 1
            return self.__hdr_len__ + \
                   self.len \
                   + n

        def __str__(self):
            if self.type & 0x80:
                length_str = None
            else:
                if self.type == USER_TO_USER:
                    length_str = struct.pack('>H', self.len) 
                else:
                    length_str = struct.pack('B', self.len)
            return struct.pack('B', self.type) + \
                   length_str + \
                   self.data


if __name__ == '__main__':
    import unittest

    class H225TestCase(unittest.TestCase):
        def testPack(self):
            h = H225(self.s)
            self.failUnless(self.s == str(h))

        def testUnpack(self):
            h = H225(self.s)
            self.failUnless(h.tpkt.v == 3)
            self.failUnless(h.tpkt.rsvd == 0)
            self.failUnless(h.tpkt.len == 1041)
            self.failUnless(h.proto == 8)
            self.failUnless(h.type == SETUP)
            self.failUnless(len(h.data) == 3)

            ie = h.data[0]
            self.failUnless(ie.type == BEARER_CAPABILITY)
            self.failUnless(ie.len == 3)
            ie = h.data[1]
            self.failUnless(ie.type == DISPLAY)
            self.failUnless(ie.len == 14)
            ie = h.data[2]
            self.failUnless(ie.type == USER_TO_USER)
            self.failUnless(ie.len == 1008)

        s = '\x03\x00\x04\x11\x08\x02\x54\x2b\x05\x04\x03\x88\x93\xa5\x28\x0e\x4a\x6f\x6e\x20\x4f\x62\x65\x72\x68\x65\x69\x64\x65\x00\x7e\x03\xf0\x05\x20\xb8\x06\x00\x08\x91\x4a\x00\x04\x01\x40\x0c\x00\x4a\x00\x6f\x00\x6e\x00\x20\x00\x4f\x00\x62\x00\x65\x00\x72\x00\x68\x00\x65\x00\x69\x00\x64\x00\x65\x22\xc0\x09\x00\x00\x3d\x06\x65\x6b\x69\x67\x61\x00\x00\x14\x32\x2e\x30\x2e\x32\x20\x28\x4f\x50\x41\x4c\x20\x76\x32\x2e\x32\x2e\x32\x29\x00\x00\x00\x01\x40\x15\x00\x74\x00\x63\x00\x70\x00\x24\x00\x68\x00\x33\x00\x32\x00\x33\x00\x2e\x00\x76\x00\x6f\x00\x78\x00\x67\x00\x72\x00\x61\x00\x74\x00\x69\x00\x61\x00\x2e\x00\x6f\x00\x72\x00\x67\x00\x42\x87\x23\x2c\x06\xb8\x00\x6a\x8b\x1d\x0c\xb7\x06\xdb\x11\x9e\xca\x00\x10\xa4\x89\x6d\x6a\x00\xc5\x1d\x80\x04\x07\x00\x0a\x00\x01\x7a\x75\x30\x11\x00\x5e\x88\x1d\x0c\xb7\x06\xdb\x11\x9e\xca\x00\x10\xa4\x89\x6d\x6a\x82\x2b\x0e\x30\x40\x00\x00\x06\x04\x01\x00\x4c\x10\x09\x00\x00\x3d\x0f\x53\x70\x65\x65\x78\x20\x62\x73\x34\x20\x57\x69\x64\x65\x36\x80\x11\x1c\x00\x01\x00\x98\xa0\x26\x41\x13\x8a\x00\x98\xa0\x26\x41\x13\x8b\x26\x00\x00\x64\x0c\x10\x09\x00\x00\x3d\x0f\x53\x70\x65\x65\x78\x20\x62\x73\x34\x20\x57\x69\x64\x65\x36\x80\x0b\x0d\x00\x01\x00\x98\xa0\x26\x41\x13\x8b\x00\x2a\x40\x00\x00\x06\x04\x01\x00\x4c\x10\x09\x00\x00\x3d\x09\x69\x4c\x42\x43\x2d\x31\x33\x6b\x33\x80\x11\x1c\x00\x01\x00\x98\xa0\x26\x41\x13\x8a\x00\x98\xa0\x26\x41\x13\x8b\x20\x00\x00\x65\x0c\x10\x09\x00\x00\x3d\x09\x69\x4c\x42\x43\x2d\x31\x33\x6b\x33\x80\x0b\x0d\x00\x01\x00\x98\xa0\x26\x41\x13\x8b\x00\x20\x40\x00\x00\x06\x04\x01\x00\x4e\x0c\x03\x00\x83\x00\x80\x11\x1c\x00\x01\x00\x98\xa0\x26\x41\x13\x8a\x00\x98\xa0\x26\x41\x13\x8b\x16\x00\x00\x66\x0e\x0c\x03\x00\x83\x00\x80\x0b\x0d\x00\x01\x00\x98\xa0\x26\x41\x13\x8b\x00\x4b\x40\x00\x00\x06\x04\x01\x00\x4c\x10\xb5\x00\x53\x4c\x2a\x02\x00\x00\x00\x00\x00\x40\x01\x00\x00\x40\x01\x02\x00\x08\x00\x00\x00\x00\x00\x31\x00\x01\x00\x40\x1f\x00\x00\x59\x06\x00\x00\x41\x00\x00\x00\x02\x00\x40\x01\x00\x00\x80\x11\x1c\x00\x01\x00\x98\xa0\x26\x41\x13\x8a\x00\x98\xa0\x26\x41\x13\x8b\x41\x00\x00\x67\x0c\x10\xb5\x00\x53\x4c\x2a\x02\x00\x00\x00\x00\x00\x40\x01\x00\x00\x40\x01\x02\x00\x08\x00\x00\x00\x00\x00\x31\x00\x01\x00\x40\x1f\x00\x00\x59\x06\x00\x00\x41\x00\x00\x00\x02\x00\x40\x01\x00\x00\x80\x0b\x0d\x00\x01\x00\x98\xa0\x26\x41\x13\x8b\x00\x32\x40\x00\x00\x06\x04\x01\x00\x4c\x10\x09\x00\x00\x3d\x11\x53\x70\x65\x65\x78\x20\x62\x73\x34\x20\x4e\x61\x72\x72\x6f\x77\x33\x80\x11\x1c\x00\x01\x00\x98\xa0\x26\x41\x13\x8a\x00\x98\xa0\x26\x41\x13\x8b\x28\x00\x00\x68\x0c\x10\x09\x00\x00\x3d\x11\x53\x70\x65\x65\x78\x20\x62\x73\x34\x20\x4e\x61\x72\x72\x6f\x77\x33\x80\x0b\x0d\x00\x01\x00\x98\xa0\x26\x41\x13\x8b\x00\x1d\x40\x00\x00\x06\x04\x01\x00\x4c\x60\x1d\x80\x11\x1c\x00\x01\x00\x98\xa0\x26\x41\x13\x8a\x00\x98\xa0\x26\x41\x13\x8b\x13\x00\x00\x69\x0c\x60\x1d\x80\x0b\x0d\x00\x01\x00\x98\xa0\x26\x41\x13\x8b\x00\x1d\x40\x00\x00\x06\x04\x01\x00\x4c\x20\x1d\x80\x11\x1c\x00\x01\x00\x98\xa0\x26\x41\x13\x8a\x00\x98\xa0\x26\x41\x13\x8b\x13\x00\x00\x6a\x0c\x20\x1d\x80\x0b\x0d\x00\x01\x00\x98\xa0\x26\x41\x13\x8b\x00\x01\x00\x01\x00\x01\x00\x01\x00\x81\x03\x02\x80\xf8\x02\x70\x01\x06\x00\x08\x81\x75\x00\x0b\x80\x13\x80\x01\xf4\x00\x01\x00\x00\x01\x00\x00\x01\x00\x00\x0c\xc0\x01\x00\x01\x80\x0b\x80\x00\x00\x20\x20\x09\x00\x00\x3d\x0f\x53\x70\x65\x65\x78\x20\x62\x73\x34\x20\x57\x69\x64\x65\x36\x80\x00\x01\x20\x20\x09\x00\x00\x3d\x09\x69\x4c\x42\x43\x2d\x31\x33\x6b\x33\x80\x00\x02\x24\x18\x03\x00\xe6\x00\x80\x00\x03\x20\x20\xb5\x00\x53\x4c\x2a\x02\x00\x00\x00\x00\x00\x40\x01\x00\x00\x40\x01\x02\x00\x08\x00\x00\x00\x00\x00\x31\x00\x01\x00\x40\x1f\x00\x00\x59\x06\x00\x00\x41\x00\x00\x00\x02\x00\x40\x01\x00\x00\x80\x00\x04\x20\x20\x09\x00\x00\x3d\x11\x53\x70\x65\x65\x78\x20\x62\x73\x34\x20\x4e\x61\x72\x72\x6f\x77\x33\x80\x00\x05\x20\xc0\xef\x80\x00\x06\x20\x40\xef\x80\x00\x07\x08\xe0\x03\x51\x00\x80\x01\x00\x80\x00\x08\x08\xd0\x03\x51\x00\x80\x01\x00\x80\x00\x09\x83\x01\x50\x80\x00\x0a\x83\x01\x10\x80\x00\x0b\x83\x01\x40\x00\x80\x01\x03\x06\x00\x00\x00\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00\x06\x01\x00\x07\x00\x08\x00\x00\x09\x01\x00\x0a\x00\x0b\x07\x01\x00\x32\x80\xa6\xff\x4c\x02\x80\x01\x80'

    unittest.main()



# $Id: __init__.py 89 2013-05-23 01:31:22Z andrewflnr@gmail.com $

"""fast, simple packet creation and parsing."""

__author__ = 'Dug Song <dugsong@monkey.org>'
__copyright__ = 'Copyright (c) 2004 Dug Song'
__license__ = 'BSD'
__url__ = 'http://dpkt.googlecode.com/'
__version__ = '1.8'

from dpkt import *

import ah
import aim
import arp
import asn1
import bgp
import cdp
import dhcp
import diameter
import dns
import dtp
import esp
import ethernet
import gre
import gzip
import h225
import hsrp
import http
import icmp
import icmp6
import ieee80211
import igmp
import ip
import ip6
import ipx
import llc
import loopback
import mrt
import netbios
import netflow
import ntp
import ospf
import pcap
import pim
import pmap
import ppp
import pppoe
import qq
import radiotap
import radius
import rfb
import rip
import rpc
import rtp
import rx
import sccp
import sctp
import sip
import sll
import smb
import ssl
import stp
import stun
import tcp
import telnet
import tftp
import tns
import tpkt
import udp
import vrrp
import yahoo
